@@ -4,16 +4,18 @@
 *                                                                              *
 *                              C O L E D I T . C                               *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: coledit.c,v 1.1 1996/12/10 18:40:03 jrh Exp $
+* $Id: coledit.c,v 1.2 1998/01/26 00:47:11 jrh Exp jrh $
 * $Log: coledit.c,v $
+* Revision 1.2  1998/01/26 00:47:11  jrh
+* Release 2.1
+*
 * Revision 1.1  1996/12/10  18:40:03  jrh
 * Initial revision
 *
-*
 */
 #include<stdlib.h>
 #include<X11/Intrinsic.h>
@@ -45,9 +47,9 @@ void GetColor(Widget, caddr_t, XmToggleButtonCallbackStruct *);
 void SetColor(Widget, float *, XmScrollBarCallbackStruct *);
 void getRGBColor(Widget, Pixel, float *, float *, float *);
 void storeColor(Pixel, struct WINDOW *, int);
+void SetSmooth(Widget, int *, XmToggleButtonCallbackStruct *);
 
 extern Widget CreatePushButtonRow(Widget, struct PushButtonRow *, int);
-extern void GetInsWeight(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void MapBox(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void GetMessageBoxButton(Widget, XtPointer, caddr_t);
 extern Pixel getPixel(int);
@@ -68,16 +70,16 @@ extern struct ORBITAL *orbitals;
 extern struct BASISSET *basisset;
 extern Pixel stdcol[9], historyColor;
 extern float historyColor_rgb[4];
-extern int rgbMode, na, ne, nmodes, nhist, debug;
+extern int rgbMode, na, ne, nmodes, nhist, debug, projectionMode;
 
 static Widget dialog, red, green, blue, button[9];
 static Pixel pixelSave;
 static float rSave, gSave, bSave;
-static int window, where;
+static int window, where, smoothSave;
 
 void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
 {
-  Widget board, form, form1;
+  Widget board, form, form1, form2=(Widget)0, smoothRed, smoothGreen, smoothBlue;
   Pixmap selected, unselected;
   Pixel p;
   Arg args[3];
@@ -105,11 +107,12 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
 
   /* This function creates the dialog for the color editor */
 
-  setMenuItem(VIEWER, 15, False);
+  setMenuItem(VIEWER, 16, False);
+  setMenuItem(VIEWER, 17, False);
   if (windows[SPECTRUM].widget != 0)
   {
-    setMenuItem(SPECTRUM, 5, False);
-    setMenuItem(SPECTRUM, 6, False);
+    setMenuItem(SPECTRUM, 7, False);
+    setMenuItem(SPECTRUM, 8, False);
   }
   if (windows[MO].widget != 0)
   {
@@ -131,7 +134,7 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
     XtSetArg(args[2], XmNmwmDecorations, MWM_DECOR_RESIZEH | MWM_DECOR_TITLE);
   else
     XtSetArg(args[2], (char *)NULL, 0);
-  dialog=XmCreateDialogShell(topShell, "colorEditor_popup", args, XtNumber(args));
+  dialog=XmCreateDialogShell(topShell, "colorEditor", args, XtNumber(args));
   board=XtVaCreateWidget("colorEditor", xmFormWidgetClass, dialog,
                          XmNautoUnmanage, False,
                          XmNdefaultPosition, False,
@@ -165,7 +168,7 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
                        getRGBColor(windows[window].widget, windows[window].background, &r, &g, &b);
                        pixelSave=windows[window].background;
                      }
-			   break;
+                     break;
     case FOREGROUND: if (rgbMode)
                      {
                        r=windows[window].foreground_rgb[0];
@@ -178,23 +181,23 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
                        getRGBColor(windows[window].widget, windows[window].foreground, &r, &g, &b);
                        pixelSave=windows[window].foreground;
                      }
-			   break;
+                     break;
     case GNORM:      if (rgbMode)
-		 	   {
-			     r=historyColor_rgb[0];
-			     g=historyColor_rgb[1];
-			     b=historyColor_rgb[2];
-			     pixelSave=-1;
+                      {
+                       r=historyColor_rgb[0];
+                       g=historyColor_rgb[1];
+                       b=historyColor_rgb[2];
+                       pixelSave=-1;
                      }
-			   else
-			   {
+                     else
+                     {
                        getRGBColor(windows[window].widget, historyColor, &r, &g, &b);
                        pixelSave=historyColor;
-			   }
-			   break;
+                     }
+                     break;
   }
   j=WHITE;
-  for (i=0; i<8; i++)
+  for (i=0; i<=MAGENTA; i++)
   {
     sprintf(buttonName, "button%1d", i);
     p=getPixel(i);
@@ -213,12 +216,40 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
     if (r1 == r && g1 == g && b1 == b) XtVaSetValues(button[i], XmNset, True, NULL);
     j=BLACK;
   }
+  if (rgbMode && where == BACKGROUND && window == VIEWER)
+  {
+    form2=XtVaCreateWidget("controlarea2", xmRowColumnWidgetClass, form,
+                           XmNorientation, XmHORIZONTAL,
+                           XmNtopAttachment, XmATTACH_WIDGET,
+                           XmNtopWidget, form1,
+                           XmNleftAttachment, XmATTACH_FORM,
+                           XmNrightAttachment, XmATTACH_FORM,
+                           NULL);
+    smoothRed=XtVaCreateManagedWidget("smoothRed", xmToggleButtonWidgetClass, form2,
+                                      XtVaTypedArg, XmNforeground, XmRString, "Red", 4,
+                                      NULL);
+    XtAddCallback(smoothRed, XmNvalueChangedCallback, (XtCallbackProc)SetSmooth,
+			(XtPointer)1);
+    if (windows[window].smooth & 1) XtVaSetValues(smoothRed, XmNset, True, NULL);
+    smoothGreen=XtVaCreateManagedWidget("smoothGreen", xmToggleButtonWidgetClass, form2,
+                                        XtVaTypedArg, XmNforeground, XmRString, "Green", 6,
+                                        NULL);
+    XtAddCallback(smoothGreen, XmNvalueChangedCallback, (XtCallbackProc)SetSmooth,
+			(XtPointer)2);
+    if (windows[window].smooth & 2) XtVaSetValues(smoothGreen, XmNset, True, NULL);
+    smoothBlue=XtVaCreateManagedWidget("smoothBlue", xmToggleButtonWidgetClass, form2,
+                                       XtVaTypedArg, XmNforeground, XmRString, "Blue", 5,
+                                        NULL);
+    XtAddCallback(smoothBlue, XmNvalueChangedCallback, (XtCallbackProc)SetSmooth,
+			(XtPointer)4);
+    if (windows[window].smooth & 4) XtVaSetValues(smoothBlue, XmNset, True, NULL);
+  }
   rSave=r;
   gSave=g;
   bSave=b;
   red=XtVaCreateManagedWidget("red", xmScaleWidgetClass, form,
                               XmNorientation, XmHORIZONTAL,
-/*                            XtVaTypedArg, XmNtroughColor, XmRString, "Red", 4, */
+                              XtVaTypedArg, XmNtroughColor, XmRString, "Red", 4,
                               XtVaTypedArg, XmNforeground, XmRString, "Red", 4,
                               XmNvalue, (int)(1000.*r),
                               XmNminimum, 0,
@@ -232,17 +263,17 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
   switch (where)
   {
     case FOREGROUND: c=&windows[window].foreground_rgb[0];
-			   break;
+                     break;
     case BACKGROUND: c=&windows[window].background_rgb[0];
-			   break;
+                     break;
     case GNORM:      c=&historyColor_rgb[0];
-			   break;
+                     break;
   }
   XtAddCallback(red, XmNvalueChangedCallback, (XtCallbackProc)SetColor, c);
   XtAddCallback(red, XmNdragCallback, (XtCallbackProc)SetColor, c);
   green=XtVaCreateManagedWidget("green", xmScaleWidgetClass, form,
                                 XmNorientation, XmHORIZONTAL,
-/*                              XtVaTypedArg, XmNtroughColor, XmRString, "Green", 6, */
+                                XtVaTypedArg, XmNtroughColor, XmRString, "Green", 6,
                                 XtVaTypedArg, XmNforeground, XmRString, "Green", 6,
                                 XmNvalue, (int)(1000.*g),
                                 XmNminimum, 0,
@@ -256,17 +287,17 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
   switch (where)
   {
     case FOREGROUND: c=&windows[window].foreground_rgb[1];
-			   break;
+                     break;
     case BACKGROUND: c=&windows[window].background_rgb[1];
-			   break;
+                     break;
     case GNORM:      c=&historyColor_rgb[1];
-			   break;
+                     break;
   }
   XtAddCallback(green, XmNvalueChangedCallback, (XtCallbackProc)SetColor, c);
   XtAddCallback(green, XmNdragCallback, (XtCallbackProc)SetColor, c);
   blue=XtVaCreateManagedWidget("blue", xmScaleWidgetClass, form,
                                XmNorientation, XmHORIZONTAL,
-/*                             XtVaTypedArg, XmNtroughColor, XmRString, "Blue", 5, */
+                               XtVaTypedArg, XmNtroughColor, XmRString, "Blue", 5,
                                XtVaTypedArg, XmNforeground, XmRString, "Blue", 5,
                                XmNvalue, (int)(1000.*b),
                                XmNminimum, 0,
@@ -280,16 +311,18 @@ void colorEditor(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
   switch (where)
   {
     case FOREGROUND: c=&windows[window].foreground_rgb[2];
-			   break;
+                     break;
     case BACKGROUND: c=&windows[window].background_rgb[2];
-			   break;
+                     break;
     case GNORM:      c=&historyColor_rgb[2];
-			   break;
+                     break;
   }
   XtAddCallback(blue, XmNvalueChangedCallback, (XtCallbackProc)SetColor, c);
   XtAddCallback(blue, XmNdragCallback, (XtCallbackProc)SetColor, c);
 
   XtManageChild(form1);
+  if (rgbMode && where == BACKGROUND && window == VIEWER) XtManageChild(form2);
+
 
   CreatePushButtonRow(form, buttons, 2);
   XtAddCallback(dialog, XmNpopupCallback, (XtCallbackProc)MapBox, (XmAnyCallbackStruct *)NULL);
@@ -307,21 +340,22 @@ void colorEditorExit(Widget widget, caddr_t which, XmPushButtonCallbackStruct *d
   {
     switch (where)
     {
-	case BACKGROUND: windows[window].background=pixelSave;
+      case BACKGROUND: windows[window].background=pixelSave;
                        windows[window].background_rgb[0]=rSave;
                        windows[window].background_rgb[1]=gSave;
                        windows[window].background_rgb[2]=bSave;
-			     break;
+                       windows[window].smooth=smoothSave;
+                       break;
       case FOREGROUND: windows[window].foreground=pixelSave;
                        windows[window].foreground_rgb[0]=rSave;
                        windows[window].foreground_rgb[1]=gSave;
                        windows[window].foreground_rgb[2]=bSave;
-			     break;
+                       break;
       case GNORM:      historyColor=pixelSave;
-			     historyColor_rgb[0]=rSave;
-			     historyColor_rgb[1]=gSave;
-			     historyColor_rgb[2]=bSave;
-			     break;
+                       historyColor_rgb[0]=rSave;
+                       historyColor_rgb[1]=gSave;
+                       historyColor_rgb[2]=bSave;
+                       break;
     }
     if (!rgbMode)
     {
@@ -337,11 +371,12 @@ void colorEditorExit(Widget widget, caddr_t which, XmPushButtonCallbackStruct *d
     }
     redraw(window);
   }
-  setMenuItem(VIEWER, 15, True);
+  if (projectionMode != ORTHO) setMenuItem(VIEWER, 16, True);
+  setMenuItem(VIEWER, 17, True);
   if (windows[SPECTRUM].widget != 0)
   {
-    setMenuItem(SPECTRUM, 5, True);
-    setMenuItem(SPECTRUM, 6, True);
+    setMenuItem(SPECTRUM, 7, True);
+    setMenuItem(SPECTRUM, 8, True);
   }
   if (windows[MO].widget != 0)
   {
@@ -366,21 +401,21 @@ void GetColor(Widget widget, caddr_t which, XmToggleButtonCallbackStruct *data)
     getRGBColor(widget, stdcol[(int)which], &r, &g, &b);
     switch (where)
     {
-	case BACKGROUND: windows[window].background_rgb[0]=r;
+      case BACKGROUND: windows[window].background_rgb[0]=r;
                        windows[window].background_rgb[1]=g;
                        windows[window].background_rgb[2]=b;
                        c.pixel=windows[window].background;
-			     break;
+                       break;
       case FOREGROUND: windows[window].foreground_rgb[0]=r;
                        windows[window].foreground_rgb[1]=g;
                        windows[window].foreground_rgb[2]=b;
                        c.pixel=windows[window].foreground;
-			     break;
+                       break;
       case GNORM:      historyColor_rgb[0]=r;
                        historyColor_rgb[1]=g;
                        historyColor_rgb[2]=b;
-			     c.pixel=historyColor;
-			     break;
+                       c.pixel=historyColor;
+                       break;
     }
     XtVaSetValues(red,   XmNvalue, (int)(1000.*r), NULL);
     XtVaSetValues(green, XmNvalue, (int)(1000.*g), NULL);
@@ -414,17 +449,17 @@ void SetColor(Widget widget, float *color, XmScrollBarCallbackStruct *data)
                      c.red=(unsigned short)(windows[window].background_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(windows[window].background_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(windows[window].background_rgb[2]*65535.+0.5);
-			   break;
+                     break;
     case FOREGROUND: c.pixel=windows[window].foreground;
                      c.red=(unsigned short)(windows[window].foreground_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(windows[window].foreground_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(windows[window].foreground_rgb[2]*65535.+0.5);
-			   break;
+                     break;
     case GNORM:      c.pixel=historyColor;
                      c.red=(unsigned short)(historyColor_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(historyColor_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(historyColor_rgb[2]*65535.+0.5);
-			   break;
+                     break;
   }
   if (!rgbMode)
   {
@@ -434,7 +469,7 @@ void SetColor(Widget widget, float *color, XmScrollBarCallbackStruct *data)
                   NULL);
     XStoreColor(XtDisplay(windows[window].widget), colormap, &c);
   }
-  for (i=0; i<SKYBLUE; i++)
+  for (i=0; i<=MAGENTA; i++)
   {
     XtVaSetValues(button[i], XmNset, False, NULL);
     getRGBColor(button[i], stdcol[i], &r, &g, &b);
@@ -455,7 +490,7 @@ void getRGBColor(Widget widget, Pixel pixel, float *r, float *g, float *b)
   XColor color;
 
   display=XtDisplay(widget);
-  XtVaGetValues(widget, XmNcolormap, &colormap, NULL);
+  XtVaGetValues(XtParent(widget), XmNcolormap, &colormap, NULL);
   color.pixel=pixel;
   XQueryColor(display, colormap, &color);
   *r=(float)(color.red)/65535.;
@@ -465,7 +500,8 @@ void getRGBColor(Widget widget, Pixel pixel, float *r, float *g, float *b)
 
 void storeColor(Pixel pixel, struct WINDOW *window, int where)
 {
-  Colormap colormap;
+  Display *display;
+  Colormap colormap, defaultColormap;
   XColor c;
 
   switch (where)
@@ -476,8 +512,18 @@ void storeColor(Pixel pixel, struct WINDOW *window, int where)
     case FOREGROUND: getRGBColor(window->widget, pixel, &(window->foreground_rgb[0]),
                                  &(window->foreground_rgb[1]), &(window->foreground_rgb[2]));
                      break;
-    case GNORM:      getRGBColor(window->widget, pixel, &historyColor_rgb[0], &historyColor_rgb[1],
-					   &historyColor_rgb[2]);
+    case GNORM:      display=XtDisplay(window->widget);
+                     XtVaGetValues(window->widget, XmNcolormap, &colormap, NULL);
+                     if (rgbMode)
+                     {
+                       defaultColormap=DefaultColormap(display, DefaultScreen(display));
+                       if (defaultColormap != colormap) colormap=defaultColormap;
+                     }
+                     c.pixel=pixel;
+                     XQueryColor(display, colormap, &c);
+                     historyColor_rgb[0]=(float)(c.red)/65535.;
+                     historyColor_rgb[1]=(float)(c.green)/65535.;
+                     historyColor_rgb[2]=(float)(c.blue)/65535.;
                      break;
   }
   if (rgbMode) return;
@@ -489,17 +535,17 @@ void storeColor(Pixel pixel, struct WINDOW *window, int where)
                      c.red=(unsigned short)(window->background_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(window->background_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(window->background_rgb[2]*65535.+0.5);
-  			   break;
+                     break;
     case FOREGROUND: c.pixel=window->foreground;
                      c.red=(unsigned short)(window->foreground_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(window->foreground_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(window->foreground_rgb[2]*65535.+0.5);
-			   break;
+                     break;
     case GNORM:      c.pixel=historyColor;
                      c.red=(unsigned short)(historyColor_rgb[0]*65535.+0.5);
                      c.green=(unsigned short)(historyColor_rgb[1]*65535.+0.5);
                      c.blue=(unsigned short)(historyColor_rgb[2]*65535.+0.5);
-		  	   break;
+                     break;
   }
   XStoreColor(XtDisplay(window->widget), colormap, &c);
 }
@@ -642,7 +688,7 @@ void allocWindowColors(void)
   Display *display;
   Colormap colormap;
   Pixel pixel;
-  unsigned long ncolors=1, *pixels;
+  unsigned long ncolors=2, *pixels;
   char *word;
   register int i=0;
 
@@ -651,12 +697,14 @@ void allocWindowColors(void)
     XtVaGetValues(windows[VIEWER].widget, XmNbackground, &pixel, NULL);
     getRGBColor(windows[VIEWER].widget, pixel, &(windows[VIEWER].background_rgb[0]),
                 &(windows[VIEWER].background_rgb[1]), &(windows[VIEWER].background_rgb[2]));
+    XtVaGetValues(windows[VIEWER].widget, XmNforeground, &pixel, NULL);
+    getRGBColor(windows[VIEWER].widget, pixel, &(windows[VIEWER].foreground_rgb[0]),
+                &(windows[VIEWER].foreground_rgb[1]), &(windows[VIEWER].foreground_rgb[2]));
   }
   else
   {
     display=XtDisplay(windows[VIEWER].widget);
     XtVaGetValues(windows[VIEWER].widget, XmNcolormap, &colormap, NULL);
-
     if (nmodes != 0)      ncolors+=2;
     if (nhist != 0)       ncolors+=3;
     if (orbitals != NULL) ncolors+=2;
@@ -668,6 +716,7 @@ void allocWindowColors(void)
       exit(-1);
     }
     windows[VIEWER].background=pixels[i++];
+    windows[VIEWER].foreground=pixels[i++];
     if (nmodes != 0)
     {
       windows[SPECTRUM].background=pixels[i++];
@@ -677,7 +726,7 @@ void allocWindowColors(void)
     {
       windows[HISTORY].background=pixels[i++];
       windows[HISTORY].foreground=pixels[i++];
-	historyColor=pixels[i++];
+      historyColor=pixels[i++];
     }
     if (orbitals != NULL)
     {
@@ -687,5 +736,14 @@ void allocWindowColors(void)
     free((void *)pixels);
     XtVaGetValues(windows[VIEWER].widget, XmNbackground, &pixel, NULL);
     storeColor(pixel, &windows[VIEWER], BACKGROUND);
+    XtVaGetValues(windows[VIEWER].widget, XmNforeground, &pixel, NULL);
+    storeColor(pixel, &windows[VIEWER], FOREGROUND);
   }
 }
+
+void SetSmooth(Widget widget, int *which, XmToggleButtonCallbackStruct *data)
+{
+  if (data->set) windows[window].smooth |= (int)which;
+  else           windows[window].smooth &= ~(int)which;
+  redraw(window);
+}
