@@ -4,19 +4,24 @@
 *                                                                              *
 *                                 D R A W . C                                  *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: draw.c,v 1.1 1996/12/10 18:40:22 jrh Exp $
+* $Id: draw.c,v 1.2 1998/01/26 00:47:27 jrh Exp jrh $
 * $Log: draw.c,v $
+* Revision 1.2  1998/01/26 00:47:27  jrh
+* Release 2.1
+*
 * Revision 1.1  1996/12/10  18:40:22  jrh
 * Initial revision
 *
-*
 */
+#include<limits.h>
 #include<math.h>
 #include<stdio.h>
+#include<stdlib.h>
+#include<time.h>
 #include<X11/Xlib.h>
 #include<X11/Intrinsic.h>
 #include<X11/StringDefs.h>
@@ -31,47 +36,57 @@
 #endif
 #include "viewmol.h"
 
-void wireModel(Dimension);
-void stickModel(int);
-void balls(double);
-void setAtomProperties(struct ELEMENT *);
+int renderMolecule(int, int);
+void wireModel(Dimension, int);
+void stickModel(int, int);
+void balls(double, int);
+void setAtomProperties(struct ELEMENT *, int);
 void drawIntern(void);
 void labelAtoms(int);
-void drawTitle(Dimension, Dimension, int);
-void drawForces(void);
-void drawNormalMode(void);
+void drawAnnotation(void);
+void drawForces(int);
+void drawNormalMode(int);
 void drawArrow(double, double, double, double, double, double);
+void drawGround(void);
+void shadow(GLfloat v0[3], GLfloat v1[3], GLfloat v2[3], GLfloat vector[4]);
+void shadowColor(void);
 
 extern void setWindowColor(int, Pixel, const float *);
 extern int StringWidth(XFontStruct *, char *);
 extern int StringHeight(XFontStruct *);
 extern void cone(double, double, double, double, double, double, double, int);
 extern double dist(double, double, double, double, double, double);
-extern void (*drawBegin)(GLenum), (*drawEnd)(void), (*drawVertex3d)(double, double, double);
+extern void pixelToWorld(int, double *, double *);
+extern void (*drawBegin)(GLenum), (*drawEnd)(void);
+extern void (*drawVertex3d)(double, double, double);
 extern void (*drawNormal3d)(double, double, double);
-extern void (*drawColor4fv)(const GLfloat *), (*drawString)(char *, double, double, double, GLuint);
-extern void (*drawLineStipple)(GLint, GLushort), (*drawDisable)(GLenum);
-extern void (*drawLineWidth)(GLfloat);
+extern void (*drawColor4fv)(const GLfloat *);
 extern void (*drawSphere)(GLUquadricObj *, GLdouble, GLint, GLint);
-extern void (*drawOrtho)(GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble);
-extern void (*drawPopMatrix)(void);
+extern void (*drawString)(char *, double, double, double, double, GLuint);
+extern void (*drawLineStipple)(GLint, GLushort);
+extern void (*drawLineWidth)(GLfloat);
+extern void (*drawDisable)(GLenum);
 extern void raytracerBegin(GLenum);
 extern void manual(Widget, caddr_t, caddr_t);
 extern char *getStringResource(Widget, char *);
 extern void printDialog(Widget, caddr_t, XmAnyCallbackStruct *);
-extern void marching_cube(double, double, double, double *grid, double, int,
-                          int, int, double, int, void (*)(GLenum), void (*)(void),
-                          void (*)(double, double, double),
-                          void (*)(double, double, double), int, int);
+extern void marchingCube(double, double, double, double *grid, double, int,
+                         int, int, double, int, void (*)(GLenum), void (*)(void),
+                         void (*)(double, double, double),
+                         void (*)(double, double, double), int, int);
 extern void ellipse(double, double, double, double, double, double, double,
                     double, double, int);
 extern void cylinder(double, double, double, double, double, double, double,
-                     int, int);
+                     int, GLenum);
 extern char *bfname(int);
 extern void raytracerMaterial(struct ELEMENT *);
+extern void transformCoordinates(int, float input[4], float output[4]);
 extern void redraw(int);
 extern void *getmem(size_t, size_t);
+extern void *expmem(void *, size_t, size_t);
 extern void fremem(void **);
+extern clock_t getCPUTime(void);
+extern void scaleAnnotation(float);
 
 extern struct ATOM *atoms;
 extern struct BOND *bonds;
@@ -83,19 +98,21 @@ extern struct OPTIMIZATION *optimization;
 extern struct COORDS *history;
 extern struct ORBITAL *orbitals;
 extern struct GRIDOBJECT *gridObjects;
+extern struct ANNOTATION *annotation;
 extern Pixel stdcol[9];
 extern Widget topShell;
 extern GLfloat light0p[], light1p[];
 extern int na, nb, ne, ninternal, nmodes, ngridobjects;
 extern int label, mode, showForces, animate, existsUnitCell, showUnitCell, showInertia;
-extern int primitive, imo, ibasfu, bfatom, lights, movelight;
+extern int primitive, imo, ibasfu, bfatom, lights, moveItem, projectionMode;
+extern int shadows;
 extern double *cnm, cycle, amplitude, forceScale, wnScale;
-extern double gridres, sphereres;
+extern double gridres, sphereres, lineWidth;
 extern double tinert[3][3];
-extern float xrot, yrot, zrot;
-extern float xmov, ymov;
+extern float xrot[4], yrot[4], zrot[4];
+extern float xmov[2], ymov[2], zmov[2];
 extern int rgbMode, picking, swapBuffers, selectAtom;
-extern int iwavef, interp, debug, setins;
+extern int iwavef, interp, debug, setins, nAnnotations;
 extern double *grid, radfac, level, weight;
 extern char *formBondLength, *formBondAngle, *formTorsionAngle;
 
@@ -103,81 +120,107 @@ void drawBackground(int window, Pixel color, double depthBufferClear)
 {
   float background_rgb[4] = {0.0, 0.0, 1.0, 0.0};
   int clear=GL_COLOR_BUFFER_BIT;
-  register int i;
 
-  setWindowColor(BACKGROUND, color, windows[window].background_rgb);
-  if (depthBufferClear != 0.0)
-  {
-    glClearDepth(depthBufferClear);
-    clear|=GL_DEPTH_BUFFER_BIT;
-  }
-  glClear(clear);
-  if (color == stdcol[SKYBLUE])
+  if (windows[window].smooth)
   {
+    background_rgb[0]=windows[window].background_rgb[0];
+    background_rgb[1]=windows[window].background_rgb[1];
+    background_rgb[2]=windows[window].background_rgb[2];
+
+    glMatrixMode(GL_PROJECTION);
     glPushMatrix();
+    glLoadIdentity();
     glOrtho(0.0, 1.0, 0.0, 1.0, 0.0, 1.0);
     glShadeModel(GL_SMOOTH);
-    for (i=0; i<=50; i++)
-    {
-      background_rgb[1]=(float)(200-2*i)/255.;
-      setWindowColor(FOREGROUND, color, background_rgb);
-      glRectf(0.0, 0.02*(i-1), 1.0, 0.02*i);
-    }
+    glBegin(GL_POLYGON);
+    if (windows[window].smooth & 1) background_rgb[0]=0.7;
+    if (windows[window].smooth & 2) background_rgb[1]=0.7;
+    if (windows[window].smooth & 4) background_rgb[2]=0.7;
+    setWindowColor(FOREGROUND, color, background_rgb);
+    glVertex3f(0.0, 0.0, 0.0);
+    glVertex3f(1.0, 0.0, 0.0);
+    if (windows[window].smooth & 1) background_rgb[0]=0.3;
+    if (windows[window].smooth & 2) background_rgb[1]=0.3;
+    if (windows[window].smooth & 4) background_rgb[2]=0.3;
+    setWindowColor(FOREGROUND, color, background_rgb);
+    glVertex3f(1.0, 1.0, 0.0);
+    glVertex3f(0.0, 1.0, 0.0);
+    glEnd();
     glShadeModel(GL_FLAT);
     glPopMatrix();
+    glMatrixMode(GL_MODELVIEW);
+    clear=GL_DEPTH_BUFFER_BIT;
   }
+  else
+  {
+    setWindowColor(BACKGROUND, color, windows[window].background_rgb);
+    if (depthBufferClear != 0.0) clear|=GL_DEPTH_BUFFER_BIT;
+  }
+  if (depthBufferClear != 0.0)
+    glClearDepth(depthBufferClear);
+  if (shadows) clear|=GL_STENCIL_BUFFER_BIT;
+  glClear(clear);
 }
 
 void drawMolecule(Widget w, caddr_t client_data, GLwDrawingAreaCallbackStruct *data)
 {
-  static float xModel, yModel, zModel;
-  static float xLight0=0.0, yLight0=0.0, zLight0=0.0;
-  static float xLight1=0.0, yLight1=0.0, zLight1=0.0;
+  GLfloat light1t[4];
+  GLfloat xview, yview, zview;
   GLfloat light0c[4] = {1.0, 1.0, 1.0, 1.0};
   GLfloat light1c[4] = {1.0, 1.0, 1.0, 1.0};
-  const float black[4] = {0.0, 0.0, 0.0, 0.0};
+  GLfloat ground0[3] = {0.0, 0.0, 0.0};
+  GLfloat ground1[3] = {1.0, 0.0, 0.0};
+  GLfloat ground2[3] = {0.0, 0.0, 1.0};
+  clock_t startTime=0;
   Dimension width, height;
-  double boxf;
-  int prim=GL_LINE_LOOP;
-  register int i, found=0;
+  int renderMode;
+  register int found;
 
-  switch (movelight)
-  {
-    case -1: xModel=xrot;
-             yModel=yrot;
-             zModel=zrot;
-             break;
-    case 0:  xLight0=xrot;
-             yLight0=yrot;
-             zLight0=zrot;
-             break;
-    case 1:  xLight1=xrot;
-             yLight1=yrot;
-             zLight1=zrot;
-             break;
-  }
+  if (debug) startTime=getCPUTime();
+  glGetIntegerv(GL_RENDER_MODE, &renderMode);
+  if (renderMode == GL_RENDER)
+    GLwDrawingAreaMakeCurrent(windows[VIEWER].widget, windows[VIEWER].context);
   if (!picking)
   {
-    GLwDrawingAreaMakeCurrent(windows[VIEWER].widget, windows[VIEWER].context);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
   }
-  glOrtho(windows[VIEWER].left, windows[VIEWER].right, windows[VIEWER].bottom,
-          windows[VIEWER].top, windows[VIEWER].near, windows[VIEWER].far);
+  XtVaGetValues(windows[VIEWER].widget, XtNwidth, &width, XtNheight, &height, NULL);
+  if (projectionMode == ORTHO)
+    glOrtho(windows[VIEWER].left, windows[VIEWER].right, windows[VIEWER].bottom,
+            windows[VIEWER].top, windows[VIEWER].near, windows[VIEWER].far);
+  else
+  {
+    if (2.0*windows[VIEWER].far > zmov[VIEWPOINT])
+      zview=2.0*windows[VIEWER].far-zmov[VIEWPOINT];
+    else
+      zview=0.1;
+    yview=0.75*windows[VIEWER].near*windows[VIEWER].top/windows[VIEWER].far;
+    xview=yview*(double)(width)/(double)(height);
+    glFrustum(xview, (-xview), yview, (-yview), zview,
+              10.0*windows[VIEWER].far-zmov[VIEWPOINT]);
+    glTranslatef(xmov[VIEWPOINT], ymov[VIEWPOINT], zmov[VIEWPOINT]);
+    glRotatef(0.25*xrot[VIEWPOINT], 1.0, 0.0, 0.0);
+    glRotatef(0.25*yrot[VIEWPOINT], 0.0, 1.0, 0.0);
+    glRotatef(0.25*zrot[VIEWPOINT], 0.0, 0.0, 1.0);
+    gluLookAt(0.0, 0.0, 3.0*windows[VIEWER].far, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
+  }
   if (!picking)
   {
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
     drawBackground(VIEWER, windows[VIEWER].background, windows[VIEWER].far);
   }
+  if (nAnnotations > 0) drawAnnotation();
+  if (shadows) glPushMatrix();
   if (primitive == GLU_FILL && windows[VIEWER].mode != WIREMODEL)
   {
     if (lights & 0x1)
     {
       glPushMatrix();
-      glRotatef(xLight0, 1.0, 0.0, 0.0);
-      glRotatef(yLight0, 0.0, 1.0, 0.0);
-      glRotatef(zLight0, 0.0, 0.0, 1.0);
+      glRotatef(xrot[LIGHTNO0], 1.0, 0.0, 0.0);
+      glRotatef(yrot[LIGHTNO0], 0.0, 1.0, 0.0);
+      glRotatef(zrot[LIGHTNO0], 0.0, 0.0, 1.0);
       glLightfv(GL_LIGHT0, GL_POSITION, light0p);
       glLightfv(GL_LIGHT0, GL_DIFFUSE,  light0c);
       glPopMatrix();
@@ -185,47 +228,108 @@ void drawMolecule(Widget w, caddr_t client_data, GLwDrawingAreaCallbackStruct *d
     if (lights & 0x2)
     {
       glPushMatrix();
-      glRotatef(xLight1, 1.0, 0.0, 0.0);
-      glRotatef(yLight1, 0.0, 1.0, 0.0);
-      glRotatef(zLight1, 0.0, 0.0, 1.0);
+      glRotatef(xrot[LIGHTNO1], 1.0, 0.0, 0.0);
+      glRotatef(yrot[LIGHTNO1], 0.0, 1.0, 0.0);
+      glRotatef(zrot[LIGHTNO1], 0.0, 0.0, 1.0);
       glLightfv(GL_LIGHT1, GL_POSITION, light1p);
       glLightfv(GL_LIGHT1, GL_DIFFUSE,  light1c);
       glPopMatrix();
     }
   }
-  glTranslatef(xmov, ymov, 0.0);
-  glRotatef(xModel, 1.0, 0.0, 0.0);
-  glRotatef(yModel, 0.0, 1.0, 0.0);
-  glRotatef(zModel, 0.0, 0.0, 1.0);
+  if (projectionMode == PERSPECTIVE)
+  {
+    drawGround();
+    glEnable(GL_CLIP_PLANE0);
+  }
+  glTranslatef(xmov[MOLECULE], ymov[MOLECULE], zmov[MOLECULE]);
+  glRotatef(xrot[MOLECULE], 1.0, 0.0, 0.0);
+  glRotatef(yrot[MOLECULE], 0.0, 1.0, 0.0);
+  glRotatef(zrot[MOLECULE], 0.0, 0.0, 1.0);
+
+  found=renderMolecule((width+height)/2, TRUE);
+
+  if (projectionMode == PERSPECTIVE) glDisable(GL_CLIP_PLANE0);
+
+  if (shadows && (lights & 0x2))
+  {
+    glStencilFunc(GL_LESS, 2, 0xffffffff);
+    glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    glColor4f(0.0, 0.0, 0.0, 0.5);
+    glPopMatrix();
+    ground0[1]=windows[VIEWER].bottom;
+    ground1[1]=windows[VIEWER].bottom;
+    ground2[1]=windows[VIEWER].bottom;
+    transformCoordinates(LIGHTNO1, light1p, light1t);
+    shadow(ground0, ground1, ground2, light1t);
+    glTranslatef(xmov[MOLECULE], ymov[MOLECULE], zmov[MOLECULE]);
+    glRotatef(xrot[MOLECULE], 1.0, 0.0, 0.0);
+    glRotatef(yrot[MOLECULE], 0.0, 1.0, 0.0);
+    glRotatef(zrot[MOLECULE], 0.0, 0.0, 1.0);
+    found=renderMolecule((width+height)/2, FALSE);
+    glDisable(GL_BLEND);
+    glDisable(GL_STENCIL_TEST);
+  }
+
+  if (label) labelAtoms(LABEL);
+  if (setins) labelAtoms(INS);
+  if (swapBuffers) GLwDrawingAreaSwapBuffers(windows[VIEWER].widget);
+  if (debug)
+  {
+    startTime=getCPUTime()-startTime;
+    if (startTime != (clock_t)0)
+      printf("%.1f fps (%.2f ms per frame)\n", (double)CLK_TCK/startTime,
+  	       1000.*startTime/(double)CLK_TCK);
+    else
+	printf("> %d fps (< %.2f ms per frame)\n", CLK_TCK, 1000./(double)CLK_TCK);
+  }
+}
+
+int renderMolecule(int width, int lighting)
+{
+  double boxf;
+  const float black[4] = {0.0, 0.0, 0.0, 0.0};
+  int prim=GL_LINE_LOOP;
+  register int i, found=0;
 
-  XtVaGetValues(windows[VIEWER].widget, XtNwidth, &width, XtNheight, &height, NULL);
   if (windows[VIEWER].mode == WIREMODEL || primitive == GLU_POINT)
   {
     glShadeModel(GL_FLAT);
-    wireModel((width+height)/2);
+    wireModel(width, lighting);
   }
   if (primitive != GLU_FILL || windows[VIEWER].mode == WIREMODEL)
     glShadeModel(GL_FLAT);
   else
   {
     glShadeModel(GL_SMOOTH);
+    glCullFace(GL_BACK);
     glEnable(GL_DEPTH_TEST);
+    glEnable(GL_CULL_FACE);
     glDepthFunc(GL_LESS);
-    glEnable(GL_LIGHTING);
-    if (lights & 0x1) glEnable(GL_LIGHT0);
-    if (lights & 0x2) glEnable(GL_LIGHT1);
+    if (lighting)
+    {
+      glEnable(GL_LIGHTING);
+      if (lights & 0x1) glEnable(GL_LIGHT0);
+      if (lights & 0x2) glEnable(GL_LIGHT1);
+    }
   }
   if (windows[VIEWER].mode == STICKMODEL || windows[VIEWER].mode == BALLMODEL)
-    stickModel(FALSE);
+    stickModel(FALSE, lighting);
   if (windows[VIEWER].mode == STICKMODEL)
-    balls(-atoms[na].rad);
+  {
+    glPassThrough((GLfloat)(ADD_DEPTH+(int)(windows[VIEWER].far)));
+    balls(-atoms[na].rad, lighting);
+    glPassThrough((GLfloat)(ADD_DEPTH));
+  }
   if (windows[VIEWER].mode == BALLMODEL)
-    balls(0.5);
+    balls(0.5, lighting);
   if (windows[VIEWER].mode == CUPMODEL)
   {
-    balls(1.3);
-    stickModel(TRUE);
+    balls(1.3, lighting);
+    stickModel(TRUE, lighting);
   }
+  if (primitive == GLU_FILL) glDisable(GL_CULL_FACE);
   if ((imo != -1 || ibasfu != -1 || iwavef != ALL_OFF) && !selectAtom)
   {
 /*  if (drawBegin == glBegin)
@@ -238,7 +342,7 @@ void drawMolecule(Widget w, caddr_t client_data, GLwDrawingAreaCallbackStruct *d
               (iwavef == BASIS_IN_MO && gridObjects[i].mo == imo &&
                                         gridObjects[i].basisfunction == ibasfu) ||
               (iwavef == BASIS_FUNCTION && gridObjects[i].basisfunction == ibasfu) ||
-		  iwavef == DENSITY)
+              iwavef == DENSITY)
           {
             found=i+1;
             if (debug) printf("Found MO/basis function: %d\n", found);
@@ -261,54 +365,64 @@ void drawMolecule(Widget w, caddr_t client_data, GLwDrawingAreaCallbackStruct *d
         case GLU_FILL:  prim=GL_TRIANGLE_STRIP;
                         break;
       }
-	if (drawBegin == raytracerBegin) prim=GL_LINE_LOOP;
+      if (drawBegin == raytracerBegin) prim=GL_LINE_LOOP;
       if (primitive == GLU_FILL)
       {
         glShadeModel(GL_SMOOTH);
         glEnable(GL_DEPTH_TEST);
         glDepthFunc(GL_LESS);
-        glEnable(GL_LIGHTING);
-        glEnable(GL_LIGHT0);
-        glEnable(GL_LIGHT1);
+        if (lighting)
+        {
+          glEnable(GL_LIGHTING);
+          if (lights & 0x1) glEnable(GL_LIGHT0);
+          if (lights & 0x2) glEnable(GL_LIGHT1);
+        }
+/*      glBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+        glEnable(GL_BLEND);*/
       }
       for (i=0; i<ne; i++)
         if (!strcmp(elements[i].symbol, "Ps")) break;
-      setAtomProperties(&elements[i]);
-      marching_cube(-boxf, -boxf, -boxf, gridObjects[found-1].grid,
-                    2.0*boxf/gridres, (int)(gridres), (int)(gridres),
-                    (int)(gridres), level, prim, drawBegin, drawEnd,
-                    drawVertex3d, drawNormal3d, interp, debug);
+      setAtomProperties(&elements[i], lighting);
+      glPassThrough((GLfloat)VISIBILITY);
+      marchingCube(-boxf, -boxf, -boxf, gridObjects[found-1].grid,
+                   2.0*boxf/gridres, (int)(gridres), (int)(gridres),
+                   (int)(gridres), level, prim, drawBegin, drawEnd,
+                   drawVertex3d, drawNormal3d, interp, debug);
       if (iwavef != DENSITY)
       {
         for (i=0; i<ne; i++)
           if (!strcmp(elements[i].symbol, "Ms")) break;
-        setAtomProperties(&elements[i]);
+        setAtomProperties(&elements[i], lighting);
         level=(-level);
-        marching_cube(-boxf, -boxf, -boxf, gridObjects[found-1].grid,
-                       2.0*boxf/gridres, (int)(gridres), (int)(gridres),
-                       (int)(gridres), level, prim, drawBegin, drawEnd,
-                       drawVertex3d, drawNormal3d, interp, debug);
+        marchingCube(-boxf, -boxf, -boxf, gridObjects[found-1].grid,
+                      2.0*boxf/gridres, (int)(gridres), (int)(gridres),
+                      (int)(gridres), level, prim, drawBegin, drawEnd,
+                      drawVertex3d, drawNormal3d, interp, debug);
         level=(-level);
       }
+      glPassThrough((GLfloat)VISIBILITY);
     }
   }
   if (primitive == GLU_FILL)
   {
     glDisable(GL_DEPTH_TEST);
     glShadeModel(GL_FLAT);
-    glDisable(GL_LIGHTING);
-    if (lights & 0x1) glDisable(GL_LIGHT0);
-    if (lights & 0x2) glDisable(GL_LIGHT1);
+    if (lighting)
+    {
+      glDisable(GL_LIGHTING);
+      if (lights & 0x1) glDisable(GL_LIGHT0);
+      if (lights & 0x2) glDisable(GL_LIGHT1);
+    }
+/*  glDisable(GL_BLEND); */
   }
-
-  if (ninternal && !picking) drawIntern();
-  if (label) labelAtoms(LABEL);
-  if (setins) labelAtoms(INS);
-  if (showForces && !picking) drawForces();
+  if (showForces && !picking) drawForces(lighting);
   if (showInertia)
   {
     (*drawLineWidth)((GLfloat)1.);
-    setWindowColor(FOREGROUND, stdcol[BLACK], black);
+    if (lighting)
+      setWindowColor(FOREGROUND, stdcol[BLACK], black);
+    else
+      shadowColor();
     ellipse(0.0, 0.0, 0.0, tinert[0][0], tinert[0][1], tinert[0][2],
             tinert[1][0], tinert[1][1], tinert[1][2], 40);
     ellipse(0.0, 0.0, 0.0, tinert[1][0], tinert[1][1], tinert[1][2],
@@ -316,19 +430,21 @@ void drawMolecule(Widget w, caddr_t client_data, GLwDrawingAreaCallbackStruct *d
     ellipse(0.0, 0.0, 0.0, tinert[2][0], tinert[2][1], tinert[2][2],
             tinert[0][0], tinert[0][1], tinert[0][2], 40);
   }
-  if (mode != -1 && animate == ARROWS && cnm != NULL) drawNormalMode();
-  if (!picking) drawTitle(width, height, found);
-  if (swapBuffers) GLwDrawingAreaSwapBuffers(windows[VIEWER].widget);
+  if (mode != -1 && animate == ARROWS && cnm != NULL) drawNormalMode(lighting);
+  if (ninternal && !picking) drawIntern();
+  return(found);
 }
 
-void wireModel(Dimension width)
+void wireModel(Dimension width, int lighting)
 { 
-  struct BONDORDER *bo;
   double x, y, z;
   int first, second;
   register int i;
 
-  (*drawLineWidth)((GLfloat)width/100.);
+  if (lineWidth == 0.0)
+    (*drawLineWidth)((GLfloat)width*0.01);
+  else
+    (*drawLineWidth)((GLfloat)lineWidth);
   for (i=0; i<nb; i++)
   {
     first=bonds[i].first;
@@ -337,52 +453,82 @@ void wireModel(Dimension width)
     x=atoms[first].x+bonds[i].frac*(atoms[second].x-atoms[first].x);
     y=atoms[first].y+bonds[i].frac*(atoms[second].y-atoms[first].y);
     z=atoms[first].z+bonds[i].frac*(atoms[second].z-atoms[first].z);
-    if (picking)
+    if (abs(bonds[i].order) != 2)
     {
-      glLoadName(first+1);
-      (*drawBegin)(GL_LINES);
-    }
-    else
-      (*drawBegin)(GL_LINE_STRIP);
-    setAtomProperties(atoms[first].element);
-    (*drawVertex3d)(atoms[first].x, atoms[first].y, atoms[first].z);
-    (*drawVertex3d)(x, y, z);
-    if (picking)
-    {
-      (*drawEnd)();
-      glLoadName(second+1);
-      (*drawBegin)(GL_LINES);
-      (*drawVertex3d)(x, y, z);
-    }
-    setAtomProperties(atoms[second].element);
-    (*drawVertex3d)(atoms[second].x, atoms[second].y, atoms[second].z);
-    (*drawEnd)();
-    if (bonds[i].order != (struct BONDORDER *)NULL)
-    {
-      bo=bonds[i].order;
-      (*drawBegin)(GL_LINE_STRIP);
-      setAtomProperties(atoms[first].element);
-      (*drawVertex3d)(bo->x1, bo->y1, bo->z1);
-      x=bo->x1+bonds[i].frac*(bo->x2-bo->x1);
-      y=bo->y1+bonds[i].frac*(bo->y2-bo->y1);
-      z=bo->z1+bonds[i].frac*(bo->z2-bo->z1);
-      (*drawVertex3d)(x, y, z);
-      setAtomProperties(atoms[second].element);
-      (*drawVertex3d)(bo->x2, bo->y2, bo->z2);
-      (*drawEnd)();
-      if (bo->n == 3)
+      if (picking)
       {
-        (*drawBegin)(GL_LINE_STRIP);
-        setAtomProperties(atoms[first].element);
-        (*drawVertex3d)(bo->x3, bo->y3, bo->z3);
-        x=bo->x3+bonds[i].frac*(bo->x4-bo->x3);
-        y=bo->y3+bonds[i].frac*(bo->y4-bo->y3);
-        z=bo->z3+bonds[i].frac*(bo->z4-bo->z3);
-        (*drawVertex3d)(x, y, z);
-        setAtomProperties(atoms[second].element);
-        (*drawVertex3d)(bo->x4, bo->y4, bo->z4);
-        (*drawEnd)();
+        glLoadName(first+1);
+        (*drawBegin)(GL_LINES);
       }
+      else
+        (*drawBegin)(GL_LINE_STRIP);
+      setAtomProperties(atoms[first].element, lighting);
+      (*drawVertex3d)(atoms[first].x, atoms[first].y, atoms[first].z);
+      (*drawVertex3d)(x, y, z);
+      if (picking)
+      {
+        (*drawEnd)();
+        glLoadName(second+1);
+        (*drawBegin)(GL_LINES);
+        (*drawVertex3d)(x, y, z);
+      }
+      setAtomProperties(atoms[second].element, lighting);
+      (*drawVertex3d)(atoms[second].x, atoms[second].y, atoms[second].z);
+      (*drawEnd)();
+    }
+    if (abs(bonds[i].order) > 1)
+    {
+      if (picking)
+      {
+        glLoadName(first+1);
+        (*drawBegin)(GL_LINES);
+      }
+      else
+        (*drawBegin)(GL_LINE_STRIP);
+      setAtomProperties(atoms[first].element, lighting);
+      (*drawVertex3d)(atoms[first].x-bonds[i].x, atoms[first].y-bonds[i].y,
+			    atoms[first].z-bonds[i].z);
+      (*drawVertex3d)(x-bonds[i].x, y-bonds[i].y, z-bonds[i].z);
+      if (picking)
+      {
+        (*drawEnd)();
+        glLoadName(second+1);
+        (*drawBegin)(GL_LINES);
+        (*drawVertex3d)(x-bonds[i].x, y-bonds[i].y, z-bonds[i].z);
+      }
+      setAtomProperties(atoms[second].element, lighting);
+      (*drawVertex3d)(atoms[second].x-bonds[i].x, atoms[second].y-bonds[i].y,
+			    atoms[second].z-bonds[i].z);
+      (*drawEnd)();
+      if (bonds[i].order == (-2))
+      {
+        glEnable(GL_LINE_STIPPLE);            /* Dashed lines */
+        (*drawLineStipple)(1, 0xf0f0);
+      }
+      if (picking)
+      {
+        glLoadName(first+1);
+        (*drawBegin)(GL_LINES);
+      }
+      else
+        (*drawBegin)(GL_LINE_STRIP);
+      setAtomProperties(atoms[first].element, lighting);
+      (*drawVertex3d)(atoms[first].x+bonds[i].x, atoms[first].y+bonds[i].y,
+			    atoms[first].z+bonds[i].z);
+      (*drawVertex3d)(x+bonds[i].x, y+bonds[i].y, z+bonds[i].z);
+      if (picking)
+      {
+        (*drawEnd)();
+        glLoadName(second+1);
+        (*drawBegin)(GL_LINES);
+        (*drawVertex3d)(x+bonds[i].x, y+bonds[i].y, z+bonds[i].z);
+      }
+      setAtomProperties(atoms[second].element, lighting);
+      (*drawVertex3d)(atoms[second].x+bonds[i].x, atoms[second].y+bonds[i].y,
+			    atoms[second].z+bonds[i].z);
+      (*drawEnd)();
+      if (bonds[i].order == (-2))
+        glDisable(GL_LINE_STIPPLE);
     }
   }
   x=windows[VIEWER].far/100.;
@@ -392,7 +538,7 @@ void wireModel(Dimension width)
     if (atoms[i].nbonds == 0)
     {
       if (picking) glLoadName(i+1);
-      setAtomProperties(atoms[i].element);
+      setAtomProperties(atoms[i].element, lighting);
       (*drawBegin)(GL_LINES);
       (*drawVertex3d)(atoms[i].x-x, atoms[i].y, atoms[i].z);
       (*drawVertex3d)(atoms[i].x+x, atoms[i].y, atoms[i].z);
@@ -405,7 +551,7 @@ void wireModel(Dimension width)
   }
 }
 
-void stickModel(int onlyUnitCell)
+void stickModel(int onlyUnitCell, int lighting)
 {
   double x, y, z;
   int first, second;
@@ -427,16 +573,16 @@ void stickModel(int onlyUnitCell)
     x=atoms[first].x+bonds[i].frac*(atoms[second].x-atoms[first].x);
     y=atoms[first].y+bonds[i].frac*(atoms[second].y-atoms[first].y);
     z=atoms[first].z+bonds[i].frac*(atoms[second].z-atoms[first].z);
-    setAtomProperties(atoms[first].element);
+    setAtomProperties(atoms[first].element, lighting);
     glLoadName(first+1);
-    cylinder(atoms[first].x, atoms[first].y, atoms[first].z, x, y, z, atoms[na].rad, (int)sphereres, primitive);
-    setAtomProperties(atoms[second].element);
+    cylinder(atoms[first].x, atoms[first].y, atoms[first].z, x, y, z, atoms[na].rad, (int)sphereres, (GLenum)primitive);
+    setAtomProperties(atoms[second].element, lighting);
     glLoadName(second+1);
-    cylinder(atoms[second].x, atoms[second].y, atoms[second].z, x, y, z, atoms[na].rad, (int)sphereres, primitive);
+    cylinder(atoms[second].x, atoms[second].y, atoms[second].z, x, y, z, atoms[na].rad, (int)sphereres, (GLenum)primitive);
   }
 }
 
-void balls(double fac)
+void balls(double fac, int lighting)
 {
   GLUquadricObj *obj;
   register double rad;
@@ -444,9 +590,9 @@ void balls(double fac)
 
   (*drawLineWidth)(1.);
   obj=gluNewQuadric();
-  gluQuadricDrawStyle(obj, primitive);
-  if (primitive == GLU_FILL) gluQuadricNormals(obj, GLU_SMOOTH);
-  else                       gluQuadricNormals(obj, GLU_NONE);
+  gluQuadricDrawStyle(obj, (GLenum)primitive);
+  if (primitive == GLU_FILL) gluQuadricNormals(obj, (GLenum)GLU_SMOOTH);
+  else                       gluQuadricNormals(obj, (GLenum)GLU_NONE);
   for (i=0; i<na; i++)
   {
     if (!strncmp(atoms[i].name, "Uc", 2))
@@ -460,7 +606,7 @@ void balls(double fac)
       rad=fac*atoms[i].rad;
     glPushMatrix();
     glTranslated(atoms[i].x, atoms[i].y, atoms[i].z);
-    setAtomProperties(atoms[i].element);
+    setAtomProperties(atoms[i].element, lighting);
     glLoadName(i+1);
     (*drawSphere)(obj, rad, (int)sphereres, (int)sphereres);
     glPopMatrix();
@@ -490,9 +636,9 @@ void drawIntern(void)
                         break;
       case BONDLENGTH:  j=internals[i].atoms[0];
                         k=internals[i].atoms[1];
-                        x=(atoms[j].x+atoms[k].x)/2.0+0.05;
-                        y=(atoms[j].y+atoms[k].y)/2.0+0.05;
-                        z=(atoms[j].z+atoms[k].z)/2.0+0.05;
+                        x=(atoms[j].x+atoms[k].x)*0.5+0.05;
+                        y=(atoms[j].y+atoms[k].y)*0.5+0.05;
+                        z=(atoms[j].z+atoms[k].z)*0.5+0.05;
                         form=formBondLength;
                         (*drawBegin)(GL_LINES);
                         (*drawVertex3d)(atoms[j].x, atoms[j].y, atoms[j].z);
@@ -515,9 +661,9 @@ void drawIntern(void)
                         break;
       case TORSION:     j=internals[i].atoms[1];
                         k=internals[i].atoms[2];
-                        x=(atoms[j].x+atoms[k].x)/2.0+0.05;
-                        y=(atoms[j].y+atoms[k].y)/2.0+0.05;
-                        z=(atoms[j].z+atoms[k].z)/2.0+0.05;
+                        x=(atoms[j].x+atoms[k].x)*0.5+0.05;
+                        y=(atoms[j].y+atoms[k].y)*0.5+0.05;
+                        z=(atoms[j].z+atoms[k].z)*0.5+0.05;
                         form=formTorsionAngle;
                         (*drawBegin)(GL_LINE_STRIP);
                         j=internals[i].atoms[0];
@@ -534,7 +680,7 @@ void drawIntern(void)
                         return;
     }
     sprintf(line, form, internals[i].value);
-    (*drawString)(line, x, y, z, windows[VIEWER].GLfontId);
+    (*drawString)(line, x, y, z, 0.0, windows[VIEWER].GLfontId);
   }
   (*drawDisable)(GL_LINE_STIPPLE);
 }
@@ -554,124 +700,50 @@ void labelAtoms(int what)
       {
         case LABEL: sprintf(name, "%s%d", atoms[i].name, i+1);
                     break;
-        case INS:   sprintf(name, "%f", atoms[i].neutron_scatterfac);
+        case INS:   sprintf(name, "%f", atoms[i].neutronScatterfac);
                     break;
       }
       glLoadName(i+1);
-      (*drawString)(name, atoms[i].x+0.05, atoms[i].y+0.05, atoms[i].z+0.05, windows[VIEWER].GLfontId);
+      (*drawString)(name, atoms[i].x+0.05, atoms[i].y+0.05, atoms[i].z+0.05, 0.0, windows[VIEWER].GLfontId);
     }
   }
 }
 
-void drawTitle(Dimension w, Dimension h, int found)
+void drawAnnotation()
 {
-  const float black[4] = {0.0, 0.0, 0.0, 0.0};
-  const float blue[4] = {0.0, 0.0, 1.0, 0.0};
-  const float red[4] = {1.0, 0.0, 0.0, 0.0};
-  char line1[MAXLENLINE], line2[MAXLENLINE], *c, *d;
-  register int i, j;
+  double xpix, ypix;
+  register double h, l;
+  register int i;
 
-  if (mode != -1)
+  pixelToWorld(VIEWER, &xpix, &ypix);
+  h=(double)StringHeight(windows[VIEWER].font)*ypix;
+  for (i=0; i<nAnnotations; i++)
   {
-    c=getStringResource(topShell, "wavenumberTitle");
-    sprintf(line1, c, normal_modes[mode].sym, wnScale*normal_modes[mode].wavenumber);
-    strcpy(line2, "");
-  }
-  else if (selectAtom)
-  {
-    c=getStringResource(topShell, "selectAtomTitle");
-    strcpy(line1, c);
-    strcpy(line2, "");
-  }
-  else if (setins)
-  {
-    c=getStringResource(topShell, "selectINSTitle");
-    sprintf(line1, c, weight);
-    strcpy(line2, "");
-  }
-  else if (found)
-  {
-    if (iwavef == BASIS_FUNCTION && ibasfu != -1)
+    if (picking)
     {
-      c=getStringResource(topShell, "basisfunctionTitle");
-      sprintf(line1, c, ibasfu+1, atoms[bfatom].name, bfatom+1, bfname(ibasfu));
-      c=getStringResource(topShell, "isosurfacePlusMinus");
-      sprintf(line2, c, level, (-level));
-    }
-    else if (iwavef == BASIS_IN_MO && ibasfu != -1)
-    {
-      c=getStringResource(topShell, "basisfunctionInMOTitle");
-      sprintf(line1, c, ibasfu+1, imo+1, atoms[bfatom].name, bfatom+1,
-              orbitals[imo].coeff[ibasfu], bfname(ibasfu));
-      c=getStringResource(topShell, "isosurfacePlusMinus");
-      sprintf(line2, c, level, (-level));
-    }
-    else if (iwavef == MOLECULAR_ORBITAL && imo != -1)
-    {
-      c=getStringResource(topShell, "molecularOrbitalTitle");
-      sprintf(line1, c, imo+1, orbitals[imo].symmetry, orbitals[imo].energy);
-      c=getStringResource(topShell, "isosurfacePlusMinus");
-      sprintf(line2, c, level, (-level));
-    }
-    else if (iwavef == DENSITY)
-    {
-      c=getStringResource(topShell, "electronDensityTitle");
-      sprintf(line1, c);
-      c=getStringResource(topShell, "isosurfacePlus");
-      sprintf(line2, c, level);
-    }
-    else
-      return;
-  }
-  else if (windows[HISTORY].widget != NULL)
-  {
-    c=getStringResource(topShell, "historyTitle");
-    sprintf(line1, c, (int)cycle, optimization[(int)cycle-1].energy,
-            optimization[(int)cycle-1].gnorm);
-    line2[0]='\0';
-  }
-  else
-    return;
-
-  setWindowColor(FOREGROUND, stdcol[BLACK], black);
-  glMatrixMode(GL_PROJECTION);
-  glPushMatrix();
-  glLoadIdentity();
-  (*drawOrtho)(0.0, (double)w, 0.0, (double)h, 0.0, 1.0);
-  glMatrixMode(GL_MODELVIEW);
-  glPushMatrix();
-  glLoadIdentity();
-  i=(w-StringWidth(windows[VIEWER].font, line1))/2;
-  j=StringHeight(windows[VIEWER].font);
-  (*drawString)(line1, (double)i, (double)(h-2*j), 0.0, windows[VIEWER].GLfontId);
-  if (line2[0] != '\0')
-  {
-    i=(w-StringWidth(windows[VIEWER].font, line2))/2;
-    if ((c=strchr(line2, ' ')) != NULL) *c='\0';
-    (*drawString)(line2, (double)i, (double)(h-4*j), 0.0, windows[VIEWER].GLfontId);
-    if (c != NULL)
-    {
-      i+=StringWidth(windows[VIEWER].font, line2);
-      setWindowColor(FOREGROUND, stdcol[RED], red);
-      if ((d=strchr(c+1, ',')) != NULL) *d='\0';
-      *c=' ';
-      (*drawString)(c, (double)i, (double)(h-4*j), 0.0, windows[VIEWER].GLfontId);
-      if (d != NULL)
+      if (annotation[i].flags & EDITABLE)
       {
-        i+=StringWidth(windows[VIEWER].font, c);
-        setWindowColor(FOREGROUND, stdcol[BLUE], blue);
-        *d=',';
-        (*drawString)(d, (double)i, (double)(h-4*j), 0.0, windows[VIEWER].GLfontId);
+        setWindowColor(BACKGROUND, windows[VIEWER].background, windows[VIEWER].background_rgb);
+        glLoadName((GLuint)((i+1) | 0x80000000));
+  	  l=(double)StringWidth(windows[VIEWER].font, annotation[i].text)*xpix;
+  	  glBegin(GL_POLYGON);
+	  glVertex3d(annotation[i].x, annotation[i].y, 0.0);
+	  glVertex3d(annotation[i].x+l, annotation[i].y, 0.0);
+	  glVertex3d(annotation[i].x+l, annotation[i].y+h, 0.0);
+	  glVertex3d(annotation[i].x, annotation[i].y+h, 0.0);
+	  glEnd();
       }
     }
-  }
-  glMatrixMode(GL_PROJECTION);
-  (*drawPopMatrix)();
-  glMatrixMode(GL_MODELVIEW);
-  glPopMatrix();
+    else
+    {
+      setWindowColor(FOREGROUND, annotation[i].color, annotation[i].color_rgb);
+      (*drawString)(annotation[i].text, annotation[i].x, annotation[i].y, 0.0,
+			  0.0, windows[VIEWER].GLfontId);
+    }
+  } 
 }
 
-void drawForces(void)
+void drawForces(int lighting)
 {
   const float black[4] = {0.0, 0.0, 0.0, 0.0};
   int primitiveSave=primitive;
@@ -681,7 +753,10 @@ void drawForces(void)
     j=(na-8)*((int)cycle-1);
   else
     j=na*((int)cycle-1);
-  setWindowColor(FOREGROUND, stdcol[BLACK], black);
+  if (lighting)
+    setWindowColor(FOREGROUND, stdcol[BLACK], black);
+  else
+    shadowColor();
   (*drawLineWidth)((GLfloat)1.);
   if (drawBegin != glBegin) primitive=GLU_LINE;
   for (i=0; i<na; i++)
@@ -690,13 +765,16 @@ void drawForces(void)
   if (drawBegin != glBegin) primitive=primitiveSave;
 }
 
-void drawNormalMode(void)
+void drawNormalMode(int lighting)
 {
   const float black[4] = {0.0, 0.0, 0.0, 0.0};
   int primitiveSave=primitive;
   register int i, j;
 
-  setWindowColor(FOREGROUND, stdcol[BLACK], black);
+  if (lighting)
+    setWindowColor(FOREGROUND, stdcol[BLACK], black);
+  else
+    shadowColor();
   j=0;
   if (drawBegin != glBegin) primitive=GLU_LINE;
   for (i=0; i<na; i++)
@@ -716,7 +794,7 @@ void drawArrow(double x1, double y1, double z1, double x2, double y2, double z2)
   {
     if (length > 0.2*windows[VIEWER].top)
       cylinder(x1, y1, z1, x1+0.8*x2, y1+0.8*y2, z1+0.8*z2, 0.01*length,
-              (int)sphereres, primitive);
+              (int)sphereres, (GLenum)primitive);
     else
     {
       (*drawLineWidth)((GLfloat)1.);
@@ -730,19 +808,97 @@ void drawArrow(double x1, double y1, double z1, double x2, double y2, double z2)
   }
 }
 
-void setAtomProperties(struct ELEMENT *element)
+void drawGround(void)
+{
+  GLdouble plane[4]={0.0, 1.0, 0.0, 0.0};
+
+  setWindowColor(FOREGROUND, windows[VIEWER].foreground,
+		     windows[VIEWER].foreground_rgb);
+  if (shadows)
+  {
+    glEnable(GL_STENCIL_TEST);
+    glStencilFunc(GL_ALWAYS, 3, 0xffffffff);
+    glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
+  }
+  glPassThrough((GLfloat)(ADD_DEPTH+(int)(2*windows[VIEWER].far)));
+  glBegin(GL_POLYGON);
+  glVertex3d(10.0*windows[VIEWER].left, windows[VIEWER].bottom, 10.0*windows[VIEWER].near);
+  glVertex3d(10.0*windows[VIEWER].left, windows[VIEWER].bottom, 10.0*windows[VIEWER].far);
+  glVertex3d(10.0*windows[VIEWER].right, windows[VIEWER].bottom, 10.0*windows[VIEWER].far);
+  glVertex3d(10.0*windows[VIEWER].right, windows[VIEWER].bottom, 10.0*windows[VIEWER].near);
+  glEnd();
+  plane[3]=(-windows[VIEWER].bottom);
+  glClipPlane(GL_CLIP_PLANE0, plane);
+  glPassThrough((GLfloat)(ADD_DEPTH));
+}
+
+void shadow(GLfloat ground0[3], GLfloat ground1[3], GLfloat ground2[3], GLfloat lightpos[4])
+{
+  GLfloat vec0[3], vec1[3], plane[4];
+  GLfloat shadowMat[4][4];
+  register GLfloat dot;
+
+  /* Need 2 vectors to find cross product. */
+  vec0[0]=ground1[0]-ground0[0];
+  vec0[1]=ground1[1]-ground0[1];
+  vec0[2]=ground1[2]-ground0[2];
+
+  vec1[0]=ground2[0]-ground0[0];
+  vec1[1]=ground2[1]-ground0[1];
+  vec1[2]=ground2[2]-ground0[2];
+
+  /* find cross product to get A, B, and C of plane equation */
+  plane[0]=vec0[1]*vec1[2]-vec0[2]*vec1[1];
+  plane[1]=vec0[0]*vec1[2]-vec0[2]*vec1[0];
+  plane[2]=vec0[0]*vec1[1]-vec0[1]*vec1[0];
+  plane[3]=-(plane[0]*ground0[0]+plane[1]*ground0[1]+plane[2]*ground0[2]);
+
+  /* Find dot product between light position vector and ground plane normal. */
+  dot=plane[0]*lightpos[0]+plane[1]*lightpos[1]+plane[2]*lightpos[2]
+     +plane[3]*lightpos[3];
+
+  shadowMat[0][0]=dot-lightpos[0]*plane[0];
+  shadowMat[1][0]=(-lightpos[0]*plane[1]);
+  shadowMat[2][0]=(-lightpos[0]*plane[2]);
+  shadowMat[3][0]=(-lightpos[0]*plane[3]);
+
+  shadowMat[0][1]=(-lightpos[1]*plane[0]);
+  shadowMat[1][1]=dot-lightpos[1]*plane[1];
+  shadowMat[2][1]=(-lightpos[1]*plane[2]);
+  shadowMat[3][1]=(-lightpos[1]*plane[3]);
+
+  shadowMat[0][2]=(-lightpos[2]*plane[0]);
+  shadowMat[1][2]=(-lightpos[2]*plane[1]);
+  shadowMat[2][2]=dot-lightpos[2]*plane[2];
+  shadowMat[3][2]=(-lightpos[2]*plane[3]);
+
+  shadowMat[0][3]=(-lightpos[3]*plane[0]);
+  shadowMat[1][3]=(-lightpos[3]*plane[1]);
+  shadowMat[2][3]=(-lightpos[3]*plane[2]);
+  shadowMat[3][3]=dot-lightpos[3]*plane[3];
+
+  glMultMatrixf((const GLfloat *)shadowMat);
+}
+
+void setAtomProperties(struct ELEMENT *element, int lighting)
 {
   GLfloat v[4];
 
   if (drawBegin == raytracerBegin) raytracerMaterial(element);
+  if (!lighting)
+  {
+    shadowColor();
+    return;
+  }
 
   if (rgbMode)
   {
     v[0]=0.5*(element->dark[0]+element->light[0]);
     v[1]=0.5*(element->dark[1]+element->light[1]);
     v[2]=0.5*(element->dark[2]+element->light[2]);
-    v[3]=0.0;
+    v[3]=1.0;
   }
+
   if (primitive == GLU_FILL && (windows[VIEWER].mode != WIREMODEL ||
       !strcmp(element->symbol, "Ps") || !strcmp(element->symbol, "Ms")))
   {
@@ -769,6 +925,13 @@ void setAtomProperties(struct ELEMENT *element)
   }
 }
 
+void shadowColor(void)
+{
+  GLfloat v[4]={0.0, 0.0, 0.0, 0.5};
+
+  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, v);
+}
+
 void viewerKeyAction(KeySym keysym)
 {
   switch (keysym)
@@ -787,11 +950,13 @@ void viewerKeyAction(KeySym keysym)
                          windows[VIEWER].bottom/=1.5;
                          windows[VIEWER].right/=1.5;
                          windows[VIEWER].left/=1.5;
+				 if (nAnnotations) scaleAnnotation(1.0/1.5);
                          break;
     case XK_Down:        windows[VIEWER].top*=1.5;
                          windows[VIEWER].bottom*=1.5;
                          windows[VIEWER].right*=1.5;
                          windows[VIEWER].left*=1.5;
+				 if (nAnnotations) scaleAnnotation(1.5);
                          break;
   }
   redraw(VIEWER);
