@@ -4,16 +4,18 @@
 *                                                                              *
 *                                 I N I T . C                                  *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: init.c,v 1.1 1996/12/10 18:41:22 jrh Exp $
+* $Id: init.c,v 1.2 1998/01/26 00:48:00 jrh Exp jrh $
 * $Log: init.c,v $
+* Revision 1.2  1998/01/26 00:48:00  jrh
+* Release 2.1
+*
 * Revision 1.1  1996/12/10  18:41:22  jrh
 * Initial revision
 *
-*
 */
 #include<stdio.h>
 #include<X11/Xlib.h>
@@ -76,6 +78,8 @@ extern void wavefunctionDialog(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void drawingModeDialog(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void printDialog(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void colorEditor(Widget, caddr_t, XmAnyCallbackStruct *);
+extern void readSpectrum(Widget, caddr_t, XmAnyCallbackStruct *);
+extern void deleteSpectrum(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void quitSpectrum(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void quitHistory(Widget, caddr_t, XmAnyCallbackStruct *);
 extern void quitMODiagram(Widget, caddr_t, XmAnyCallbackStruct *);
@@ -83,23 +87,28 @@ extern Widget makeMenu(Widget, int, char *, struct MenuItem *);
 extern void doMenu(Widget, Widget, caddr_t);
 extern void zoomOut(Widget, caddr_t, caddr_t);
 extern void manual(Widget, caddr_t, caddr_t);
-extern void (*drawString)(char *, double, double, double, GLuint);
+extern void (*drawString)(char *, double, double, double, double, GLuint);
 extern void setWindowColor(int, Pixel, const float *);
 extern void getImaginaryMode(Widget, caddr_t, caddr_t);
-extern void saveMolecule(Widget, caddr_t, caddr_t);
+extern void saveMoleculeDialog(Widget, caddr_t, XmAnyCallbackStruct *);
+extern void saveConfiguration(Widget, caddr_t, caddr_t);
 extern void raytracer(Widget, caddr_t, caddr_t);
 extern void setTransition(Widget, caddr_t, XmToggleButtonCallbackStruct *);
+extern void setGeometry(int);
+extern void setBondType(Widget, caddr_t, caddr_t);
+extern void setAnnotation(Widget, int *, GLwDrawingAreaCallbackStruct *);
 
 extern struct BASISSET *basisset;
 extern struct ORBITAL *orbitals;
 extern struct NORMAL_MODE *normal_modes;
 extern struct WINDOW windows[];
 extern Pixel stdcol[9];
-extern int rgbMode;
+extern int rgbMode, projectionMode;
 extern char title[];
 extern Widget topShell;
-extern int label, showForces, nmodes, nhist, nbasfu, imag;
+extern int label, showForces, nmodes, nhist, nbasfu, imag, noutput;
 extern int existsUnitCell, showUnitCell, showInertia, swapBuffers;
+extern int annotate;
 
 #include "menudef.h"
 
@@ -125,20 +134,23 @@ int initViewer(int argc, char **argv, Widget widget)
   if (windows[VIEWER].mode == WIREMODEL)
   {
     setMenuItem(VIEWER,  0, False);
-    setMenuItem(VIEWER, 19, False);
+    setMenuItem(VIEWER, 22, False);
   }
   if (windows[VIEWER].mode == STICKMODEL) setMenuItem(VIEWER, 1, False);
   if (windows[VIEWER].mode == BALLMODEL)  setMenuItem(VIEWER, 2, False);
   if (windows[VIEWER].mode == CUPMODEL)   setMenuItem(VIEWER, 3, False);
-  if (basisset == NULL)                   setMenuItem(VIEWER, 6, False);
-  if (orbitals == NULL)                   setMenuItem(VIEWER, 7, False);
-  if (nhist < 2)                          setMenuItem(VIEWER, 8, False);
-  if (nhist == 0)                         setMenuItem(VIEWER, 9, False);
-  if (nmodes == 0)                        setMenuItem(VIEWER, 10, False);
-  if (!setAtom())                         setMenuItem(VIEWER, 11, False);
+  if (basisset == NULL)                   setMenuItem(VIEWER, 7, False);
+  if (orbitals == NULL)                   setMenuItem(VIEWER, 8, False);
+  if (nhist < 2)                          setMenuItem(VIEWER, 9, False);
+  if (nhist == 0)                         setMenuItem(VIEWER, 10, False);
+  if (nmodes == 0)                        setMenuItem(VIEWER, 11, False);
+  if (!setAtom())                         setMenuItem(VIEWER, 12, False);
+  if (projectionMode == ORTHO)            setMenuItem(VIEWER, 16, False);
+  if (noutput == 0)                       setMenuItem(VIEWER, 25, False);
   inert();
   if (!makeConnectivity(existsUnitCell)) return(FALSE);
   if (!existsUnitCell) showUnitCell=FALSE;
+  if (showUnitCell) XtVaSetValues(viewerMenu[11].widget, XmNset, True, NULL);
   blendToPicture();
   setCursor(widget, XC_top_left_arrow);
   return(TRUE);
@@ -151,7 +163,7 @@ void initSpectrum(Widget w, caddr_t client_data, caddr_t call_data)
   char t[7], *string, *label;
   register int i;
 
-  makeWindow(SPECTRUM, "spectrum", (void *)drawSpectrum);
+  makeWindow(SPECTRUM, "spectrum", (void *(*)())drawSpectrum);
   string=getStringResource(windows[SPECTRUM].widget, "font");
   windows[SPECTRUM].font=makeRasterFont(XtDisplay(windows[SPECTRUM].widget), string, &windows[SPECTRUM].GLfontId);
   if (imag != 0)
@@ -178,9 +190,10 @@ void initSpectrum(Widget w, caddr_t client_data, caddr_t call_data)
   windows[SPECTRUM].menu=makeMenu(windows[SPECTRUM].widget, XmMENU_POPUP, "spectrum_menu", spectrumMenu);
   XtAddCallback(windows[SPECTRUM].widget, XmNinputCallback, (XtCallbackProc)doMenu, windows[SPECTRUM].menu);
 #endif
-  setMenuItem(VIEWER, 10, False);
+  setMenuItem(VIEWER, 11, False);
   if (imag == 0) setMenuItem(SPECTRUM, 1, False);
-  setMenuItem(SPECTRUM, 2, False);
+  setMenuItem(SPECTRUM, 3, False);
+  setMenuItem(SPECTRUM, 4, False);
   XtVaGetValues(windows[SPECTRUM].widget, XmNbackground, &pixel, NULL);
   storeColor(pixel, &windows[SPECTRUM], BACKGROUND);
   XtVaGetValues(windows[SPECTRUM].widget, XmNforeground, &pixel, NULL);
@@ -195,7 +208,7 @@ void initHistory(Widget w, caddr_t client_data, caddr_t call_data)
   Pixel pixel;
   char *string;
 
-  makeWindow(HISTORY, "history", (void *)drawHistory);
+  makeWindow(HISTORY, "history", (void *(*)())drawHistory);
   string=getStringResource(windows[HISTORY].widget, "font");
   windows[HISTORY].font=makeRasterFont(XtDisplay(windows[HISTORY].widget), string, &windows[HISTORY].GLfontId);
 #ifndef LINUX
@@ -205,12 +218,13 @@ void initHistory(Widget w, caddr_t client_data, caddr_t call_data)
   windows[HISTORY].menu=makeMenu(windows[HISTORY].widget, XmMENU_POPUP, "history_menu", historyMenu);
   XtAddCallback(windows[HISTORY].widget, XmNinputCallback, (XtCallbackProc)doMenu, windows[HISTORY].menu);
 #endif
-  setMenuItem(VIEWER, 8, False);
+  setMenuItem(VIEWER, 9, False);
   XtVaGetValues(windows[HISTORY].widget, XmNbackground, &pixel, NULL);
   storeColor(pixel, &windows[HISTORY], BACKGROUND);
   XtVaGetValues(windows[HISTORY].widget, XmNforeground, &pixel, NULL);
   storeColor(pixel, &windows[HISTORY], FOREGROUND);
   storeColor(stdcol[GREEN], &windows[HISTORY], GNORM);
+  setGeometry(TRUE);
   redraw(VIEWER);
 }
 
@@ -219,7 +233,7 @@ void initMODiagram(Widget w, caddr_t client_data, caddr_t call_data)
   Pixel pixel;
   char *string;
 
-  makeWindow(MO, "MODiagram", (void *)drawMODiagram);
+  makeWindow(MO, "MODiagram", (void *(*)())drawMODiagram);
   string=getStringResource(windows[MO].widget, "font");
   windows[MO].font=makeRasterFont(XtDisplay(windows[MO].widget), string, &windows[MO].GLfontId);
 #ifndef LINUX
@@ -231,7 +245,7 @@ void initMODiagram(Widget w, caddr_t client_data, caddr_t call_data)
 #endif
   setMenuItem(MO, 1, False);
   setMenuItem(MO, 2, False);
-  setMenuItem(VIEWER, 7, False);
+  setMenuItem(VIEWER, 8, False);
   XtVaGetValues(windows[MO].widget, XmNbackground, &pixel, NULL);
   storeColor(pixel, &windows[MO], BACKGROUND);
   XtVaGetValues(windows[MO].widget, XmNforeground, &pixel, NULL);
@@ -256,23 +270,30 @@ void makeWindow(int which, char *name, void *drawFunc())
   XtRealizeWidget(shell);
   XtVaGetValues(drawArea, XmNheight, &height, XmNwidth, &width, NULL);
   XtVaGetValues(windows[VIEWER].widget, GLwNvisualInfo, &vi, NULL);
-#ifndef MESA
-  if (rgbMode)
-    colormap=XCreateColormap(display, XtWindow(drawArea), vi->visual, AllocNone);
-  else
-#endif
-    colormap=DefaultColormap(display, DefaultScreen(display));
+
   windows[which].widget=XtVaCreateManagedWidget("canvas", glwMDrawingAreaWidgetClass, drawArea,
                                                 GLwNvisualInfo, vi,
                                                 XmNheight, height,
                                                 XmNwidth, width,
-                                                XmNcolormap, colormap,
                                                 NULL);
+#ifndef LINUX
+  if (!rgbMode)
+  {
+#endif
+    if (vi->visual == DefaultVisual(display, DefaultScreen(display)))
+      colormap=DefaultColormap(display, DefaultScreen(display));
+    else
+      colormap=XCreateColormap(display, XtWindow(drawArea), vi->visual, AllocNone);
+    XtVaSetValues(windows[which].widget, XmNcolormap, colormap, NULL);
+#ifndef LINUX
+  }
+#endif
   windows[which].context=glXCreateContext(XtDisplay(windows[which].widget), vi, 0, GL_FALSE);
   GLwDrawingAreaMakeCurrent(windows[which].widget, windows[which].context);
   XtAddCallback(windows[which].widget, GLwNinputCallback, (XtCallbackProc)processInput, NULL);
   XtAddCallback(drawArea,              XmNresizeCallback, (XtCallbackProc)reshape, (XtPointer)which);
   XtAddCallback(windows[which].widget, GLwNexposeCallback, (XtCallbackProc)drawFunc, NULL);
+  windows[which].smooth=0;
 }
 
 void setCursor(Widget widget, unsigned int shape)
@@ -321,7 +342,12 @@ void showTitle(Dimension width, Dimension height)
   strcat(versionLine, " ");
   strcat(versionLine, VERSION);
   GLwDrawingAreaMakeCurrent(windows[VIEWER].widget, windows[VIEWER].context);
+  windows[VIEWER].background_rgb[0]=0.0;
+  windows[VIEWER].background_rgb[1]=0.0;
+  windows[VIEWER].background_rgb[2]=1.0;
+  windows[VIEWER].smooth=2;
   drawBackground(VIEWER, stdcol[SKYBLUE], 0.0);
+  windows[VIEWER].smooth=0;
   glLoadIdentity();
   gluOrtho2D((double)0.0, (double)width, (double)0.0, (double)height);
   setWindowColor(FOREGROUND, stdcol[YELLOW], yellow);
@@ -343,12 +369,12 @@ void showTitle(Dimension width, Dimension height)
       font=makeRasterFont(display, "variable", &GLfont);
   }
   w=StringWidth(font, viewmol);
-  (*drawString)(viewmol, (GLdouble)(width-w)/2.e0, 2.e0/3.e0*height, 0.0, GLfont);
+  (*drawString)(viewmol, (GLdouble)(width-w)/2.e0, 2.e0/3.e0*height, 0.0, 0.0, GLfont);
   deleteFontList(GLfont, font);
   setWindowColor(FOREGROUND, stdcol[BLACK], black);
   if (cyrillic)
   {
-    font=makeRasterFont(display, "-cronyx-helvetica-medium-r-normal--14-140-75-75-p-77-koi8r-1-14",
+    font=makeRasterFont(display, "-cronyx-helvetica-medium-r-normal--10-100-75-75-p-56-koi8r-1-10",
                         &GLfont);
     if (font == NULL)
     {
@@ -364,11 +390,11 @@ void showTitle(Dimension width, Dimension height)
       font=makeRasterFont(display, "variable", &GLfont);
   }
   w=StringWidth(font, versionLine);
-  (*drawString)(versionLine, (GLdouble)(width-w)/2.e0, height/2.e0, 0.0, GLfont);
+  (*drawString)(versionLine, (GLdouble)(width-w)/2.e0, height/2.e0, 0.0, 0.0, GLfont);
   w=StringWidth(font, by);
-  (*drawString)(by, (GLdouble)(width-w)/2.e0, height/2.5e0, 0.0, GLfont);
+  (*drawString)(by, (GLdouble)(width-w)/2.e0, height/2.5e0, 0.0, 0.0, GLfont);
   w=StringWidth(font, author);
-  (*drawString)(author, (GLdouble)(width-w)/2.e0, height/3.e0, 0.0, GLfont);
+  (*drawString)(author, (GLdouble)(width-w)/2.e0, height/3.e0, 0.0, 0.0, GLfont);
   deleteFontList(GLfont, font);
   GLwDrawingAreaSwapBuffers(windows[VIEWER].widget);
 }
@@ -421,32 +447,32 @@ void blendToPicture(void)
   {
     while (i<k)
     {
-      glRasterPos2i((GLint)(i*incx+1), (GLint)(j*incy+1));
-      glCopyPixels((GLint)(i*incx+1), (GLint)(j*incy+1), incx, incy, GL_COLOR);
+      glRasterPos2i((GLint)(i*incx), (GLint)(j*incy));
+      glCopyPixels((GLint)(i*incx), (GLint)(j*incy), incx, incy, GL_COLOR);
       i++;
     }
     while (j<k)
     {
-      glRasterPos2i((GLint)(i*incx+1), (GLint)(j*incy+1));
-      glCopyPixels((GLint)(i*incx+1), (GLint)(j*incy+1), incx, incy, GL_COLOR);
+      glRasterPos2i((GLint)(i*incx), (GLint)(j*incy));
+      glCopyPixels((GLint)(i*incx), (GLint)(j*incy), incx, incy, GL_COLOR);
       j++;
     }
     while (i>ngrid-k-1)
     {
-      glRasterPos2i((GLint)(i*incx+1), (GLint)(j*incy+1));
-      glCopyPixels((GLint)(i*incx+1), (GLint)(j*incy+1), incx, incy, GL_COLOR);
+      glRasterPos2i((GLint)(i*incx), (GLint)(j*incy));
+      glCopyPixels((GLint)(i*incx), (GLint)(j*incy), incx, incy, GL_COLOR);
       i--;
     }
     while (j>ngrid-k)
     {
-      glRasterPos2i((GLint)(i*incx+1), (GLint)(j*incy+1));
-      glCopyPixels((GLint)(i*incx+1), (GLint)(j*incy+1), incx, incy, GL_COLOR);
+      glRasterPos2i((GLint)(i*incx), (GLint)(j*incy));
+      glCopyPixels((GLint)(i*incx), (GLint)(j*incy), incx, incy, GL_COLOR);
       j--;
     }
     glXWaitGL();
   }
-  glRasterPos2i((GLint)(i*incx+1), (GLint)(j*incy+1));
-  glCopyPixels((GLint)(i*incx+1), (GLint)(j*incy+1), incx, incy, GL_COLOR);
+  glRasterPos2i((GLint)(i*incx), (GLint)(j*incy));
+  glCopyPixels((GLint)(i*incx), (GLint)(j*incy), incx, incy, GL_COLOR);
   glXWaitGL();
   glDrawBuffer(GL_BACK);
 }
