@@ -4,21 +4,28 @@
 *                                                                              *
 *                              M E N U D E F . H                               *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: menudef.h,v 1.1 1996/12/10 18:45:53 jrh Exp $
+* $Id: menudef.h,v 1.2 1998/01/26 00:48:32 jrh Exp jrh $
 * $Log: menudef.h,v $
- * Revision 1.1  1996/12/10  18:45:53  jrh
- * Initial revision
- *
+* Revision 1.2  1998/01/26 00:48:32  jrh
+* Release 2.1
+*
+* Revision 1.1  1996/12/10  18:45:53  jrh
+* Initial revision
 *
 */
 static struct MenuItem geometryMenu[] = {
   { "clear_all",  &xmPushButtonGadgetClass, clearGeometry, (XtPointer)TRUE,  NULL, NULL },
   { "clear_last", &xmPushButtonGadgetClass, clearGeometry, (XtPointer)FALSE, NULL, NULL },
   { NULL }};
+static struct MenuItem bondTypeMenu[] = {
+  { "single",      &xmPushButtonGadgetClass, setBondType, (XtPointer)SINGLE_BONDS,   NULL, NULL },
+  { "multiple",    &xmPushButtonGadgetClass, setBondType, (XtPointer)MULTIPLE_BONDS, NULL, NULL },
+  { "conjugated",  &xmPushButtonGadgetClass, setBondType, (XtPointer)CONJUGATION,    NULL, NULL },
+  { NULL }};
 static struct MenuItem viewerMenu[] = {
   { "wire_model",                &xmPushButtonGadgetClass,   setModel,           (XtPointer)WIREMODEL,  NULL, NULL },
   { "stick_model",               &xmPushButtonGadgetClass,   setModel,           (XtPointer)STICKMODEL, NULL, NULL },
@@ -26,6 +33,7 @@ static struct MenuItem viewerMenu[] = {
   { "cpk_model",                 &xmPushButtonGadgetClass,   setModel,           (XtPointer)CUPMODEL,   NULL, NULL },
   { "sep1",                      &xmSeparatorGadgetClass,    NULL,               (XtPointer)0,          NULL, NULL },
   { "geometry_menu",             &xmPushButtonGadgetClass,   NULL,               (XtPointer)0,          NULL, geometryMenu },
+  { "bondType_menu",             &xmPushButtonGadgetClass,   NULL,               (XtPointer)0,          NULL, bondTypeMenu },
   { "wave_function",             &xmPushButtonGadgetClass,   wavefunctionDialog, (XtPointer)0,          NULL, NULL },
   { "energy_level_diagram",      &xmPushButtonGadgetClass,   initMODiagram,      (XtPointer)0,          NULL, NULL },
   { "optimization_history",      &xmPushButtonGadgetClass,   initHistory,        (XtPointer)0,          NULL, NULL },
@@ -35,19 +43,24 @@ static struct MenuItem viewerMenu[] = {
   { "show_ellipsoid_of_inertia", &xmToggleButtonGadgetClass, changeBoolean,      (XtPointer)&showInertia, NULL, NULL },
   { "sep2",                      &xmSeparatorGadgetClass,    NULL,               (XtPointer)0,          NULL, NULL },
   { "drawing_modes",             &xmPushButtonGadgetClass,   drawingModeDialog,  (XtPointer)0,          NULL, NULL },
+  { "foreground_color",          &xmPushButtonGadgetClass,   colorEditor,        (XtPointer)(FOREGROUND*8+VIEWER), NULL, NULL},
   { "background_color",          &xmPushButtonGadgetClass,   colorEditor,        (XtPointer)(BACKGROUND*8+VIEWER), NULL, NULL},
   { "label_atoms",               &xmToggleButtonGadgetClass, changeBoolean,      (XtPointer)&label,     NULL, NULL },
+  { "annotate",                  &xmPushButtonGadgetClass,   setAnnotation,      (XtPointer)0,          NULL, NULL },
   { "sep3",                      &xmSeparatorGadgetClass,    NULL,               (XtPointer)0,          NULL, NULL },
   { "hardcopy",                  &xmPushButtonGadgetClass,   printDialog,        (XtPointer)VIEWER,     NULL, NULL },
   { "raytracing",                &xmPushButtonGadgetClass,   raytracer,          (XtPointer)0,          NULL, NULL },
   { "manual",                    &xmPushButtonGadgetClass,   manual,             (XtPointer)0,          NULL, NULL },
   { "sep4",                      &xmSeparatorGadgetClass,    NULL,               (XtPointer)0,          NULL, NULL },
-  { "save",                      &xmPushButtonGadgetClass,   saveMolecule,       (XtPointer)0,          NULL, NULL },
+  { "saveMolecule",              &xmPushButtonGadgetClass,   saveMoleculeDialog, (XtPointer)0,          NULL, NULL },
+  { "saveConfiguration",         &xmPushButtonGadgetClass,   saveConfiguration,  (XtPointer)0,          NULL, NULL },
   { "quit",                      &xmPushButtonGadgetClass,   ende,               (XtPointer)0,          NULL, NULL },
   { NULL } };
 static struct MenuItem spectrumMenu[] = {
   { "settings_spectrum",      &xmPushButtonGadgetClass,   spectrumDialog,   (XtPointer)0,        NULL, NULL },
   { "imaginary_wave_numbers", &xmPushButtonGadgetClass,   NULL,             (XtPointer)0,        NULL, NULL },
+  { "observed_spectrum",      &xmPushButtonGadgetClass,   readSpectrum,     (XtPointer)0,        NULL, NULL },
+  { "delete_spectrum",        &xmPushButtonGadgetClass,   deleteSpectrum,   (XtPointer)0,        NULL, NULL },
   { "zoom_out",               &xmPushButtonGadgetClass,   zoomOut,          (XtPointer)SPECTRUM, NULL, NULL },
   { "hardcopy",               &xmPushButtonGadgetClass,   printDialog,      (XtPointer)SPECTRUM, NULL, NULL },
   { "sep",                    &xmSeparatorGadgetClass,    NULL,             (XtPointer)0,        NULL, NULL },
