@@ -4,16 +4,18 @@
 *                                                                              *
 *                               M O F O R M . C                                *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: moform.c,v 1.1 1996/12/10 18:42:23 jrh Exp $
+* $Id: moform.c,v 1.2 1998/01/26 00:48:47 jrh Exp jrh $
 * $Log: moform.c,v $
+* Revision 1.2  1998/01/26 00:48:47  jrh
+* Release 2.1
+*
 * Revision 1.1  1996/12/10  18:42:23  jrh
 * Initial revision
 *
-*
 */
 #include<math.h>
 #include<X11/Intrinsic.h>
@@ -55,6 +57,7 @@ extern void MapBox(Widget, caddr_t, XmAnyCallbackStruct *);
 extern Widget CreateToggleBox(Widget, struct PushButtonRow *, int, int,
 	  		            int, int, int);
 extern void CreatePushButtonRow(Widget, struct PushButtonRow *, int);
+extern void setMenuItem(int, int, int);
 
 void MODiagramDialog(Widget widget, caddr_t dummy, XmAnyCallbackStruct *data)
 {
@@ -78,6 +81,8 @@ void MODiagramDialog(Widget widget, caddr_t dummy, XmAnyCallbackStruct *data)
 
   /* This function creates the dialog for the MO diagram window */
 
+  setMenuItem(MO, 0, False);
+
   unit_save=unit;
   denres_save=denres;
 
@@ -138,7 +143,7 @@ void MODiagramDialog(Widget widget, caddr_t dummy, XmAnyCallbackStruct *data)
     XtVaSetValues(slider, XmNvalue, (int)(denres*pow((double)10., (double)(decimals))), NULL);
   resolution.number=&denres;
   resolution.decimals=(int)decimals;
-  resolution.draw=(void *)drawMODiagram;
+  resolution.draw=(void (*)())drawMODiagram;
   XtAddCallback(slider, XmNvalueChangedCallback, (XtCallbackProc)GetSlider, &resolution);
   XtManageChild(form2);
 
@@ -156,7 +161,7 @@ void GetUnit(Widget widget, struct TOGGLE *which, XmToggleButtonCallbackStruct *
 
 void GetSlider(Widget widget, struct SLIDER *slider, XmScrollBarCallbackStruct *data)
 {
-  *(slider->number)=(double)(data->value)/pow(10.e0, slider->decimals);
+  *(slider->number)=(double)(data->value)/pow(10.0, slider->decimals);
   if (*(slider->draw) != NULL) (*(slider->draw))((Widget)0, (caddr_t)0, (caddr_t)0);
 }
 
@@ -168,5 +173,6 @@ void MODiagramDialogExit(Widget widget, caddr_t which, XmPushButtonCallbackStruc
     denres=denres_save;
   }
   XtDestroyWidget(dialog);
+  setMenuItem(MO, 0, True);
   drawMODiagram((Widget)0, (caddr_t)0, (XmAnyCallbackStruct *)0);
 }
