@@ -4,16 +4,18 @@
 *                                                                              *
 *                          M O U S E A C T I O N . C                           *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: mouseaction.c,v 1.1 1996/12/10 18:42:28 jrh Exp $
+* $Id: mouseaction.c,v 1.2 1998/01/26 00:48:50 jrh Exp jrh $
 * $Log: mouseaction.c,v $
+* Revision 1.2  1998/01/26 00:48:50  jrh
+* Release 2.1
+*
 * Revision 1.1  1996/12/10  18:42:28  jrh
 * Initial revision
 *
-*
 */
 #include<math.h>
 #include<stdio.h>
@@ -38,7 +40,7 @@
 
 void viewerMouseAction(XEvent *, int);
 void spectrumMouseAction(GLwDrawingAreaCallbackStruct *, int);
-void historyMouseAction(unsigned int, int, int, int, Dimension, Dimension);
+void historyMouseAction(unsigned int, int, int, int);
 void MODiagramMouseAction(GLwDrawingAreaCallbackStruct *, int);
 void setGeometry(int);
 void restoreGeometry(void);
@@ -65,44 +67,51 @@ extern void *expmem(void *, size_t, size_t);
 extern void fremem(void **);
 extern void calcInternal(int);
 extern void redraw(int);
-extern int messgb(Widget, int, char *, struct PushButtonRow *, int);
+extern int  messgb(Widget, int, char *, struct PushButtonRow *, int);
 extern void initZoom(GLwDrawingAreaCallbackStruct *);
-extern void endZoom(int width, int height, int x, int y);
-extern void pixwor(Dimension, Dimension, GLdouble, GLdouble, GLdouble, GLdouble,
-                   double *, double *);
-extern int makeConnectivity(int);
+extern void endZoom(Dimension, int, int);
+extern void pixelToWorld(int, double *, double *);
+extern int  makeConnectivity(int);
 extern void setMenuItem(int, int, int);
 extern void distortGeometry(double);
+extern int  makeAnnotation(int, int, float, float, int, const GLfloat *, int, char *);
+extern void deleteAnnotation(int *);
+extern void annotateWavefunction(void);
+extern void modifyAnnotation(int, int, int);
 
 extern XtAppContext app;
 extern struct WINDOW windows[4];
 extern struct ATOM *atoms;
+extern struct BOND *bonds;
+extern struct NORMAL_MODE *normal_modes;
 extern struct COORDS *history;
 extern struct INTERNAL *internals;
 extern struct ORBITAL *orbitals;
 extern struct OPTIMIZATION *optimization;
 extern struct BASISSET *basisset;
 extern double *coord, *grid, forceScale, amplitude;
-extern double cycle, weight, *cnm;
+extern double cycle, weight, *cnm, wnScale;
 extern int picking, swapBuffers, setins;
-extern int imo, imosav, bfatom;
-extern int rotateXY, rotateZ;
+extern int imo, imosav, bfatom, annotate;
+extern int rotateXY, rotateZ, moveItem;
 extern int debug, mode, needMoloch;
-extern int na, ninternal, nhist;
-extern int existsUnitCell;
+extern int na, ninternal, nhist, historyAnnotation;
+extern int existsUnitCell, normalModeAnnotation;
 extern XtAppContext app;
 extern Widget topShell;
 extern int showMenu, animate, selectAtom;
+extern Pixel stdcol[9];
 
 static int naSaved;
 
 void viewerMouseAction(XEvent *event, int state)
 {
   GLint picked;
-  static int new=TRUE, intcoord;
-  static int clicked[4];
+  static int new=TRUE, intcoord, moveItemSave;
+  static int clicked[4] = {-1, -1, -1, -1};
   register int i, j, k, l;
 
+  if (!(moveItem & 0x80000000)) moveItemSave=moveItem;
   if (event->xbutton.button == Button1 && state == TRUE)
   {
     picking=TRUE;
@@ -110,22 +119,33 @@ void viewerMouseAction(XEvent *event, int state)
     picked=doPicking(event->xbutton.x, event->xbutton.y, drawMolecule);
     picking=FALSE;
     swapBuffers=TRUE;
-    if (picked > -1)
+    if (picked != -1)
     {
       if (selectAtom)
       {
         ringBell();
-        basisDialog(picked);
+	  basisDialog(picked);
         bfatom=picked;
         return;
       }
       if (setins)
       {
         ringBell();
-        atoms[picked].neutron_scatterfac=weight;
+        atoms[picked].neutronScatterfac=weight;
         makeINSIntensity();
         return;
       }
+	if (picked & 0x80000000)
+	{
+	  if (!(event->xmotion.state & ShiftMask))
+	    modifyAnnotation((picked & 0x7fffffff), event->xbutton.x, event->xbutton.y);
+	  else
+	  {
+	    moveItem=picked;
+	    rotateXY=(-1);
+	  }
+	  return;
+	}
       if (new)
 	{
         new=FALSE;
@@ -177,8 +197,14 @@ void viewerMouseAction(XEvent *event, int state)
 
   switch (event->xbutton.button)
   {
-    case Button1:   if (state == TRUE) rotateXY=(-1);
-                    else               rotateXY=0;
+    case Button1:   if (annotate)
+			    modifyAnnotation((-1), event->xbutton.x, event->xbutton.y);
+			  else if (state == TRUE) rotateXY=(-1);
+                    else
+			  {
+			    rotateXY=0;
+			    moveItem=moveItemSave;
+                    }
                     break;
     case Button2:   if (state == TRUE) rotateZ=(-1);
                     else               rotateZ=0;
@@ -265,8 +291,9 @@ void clearGeometry(Widget widget, caddr_t all, XmAnyCallbackStruct *data)
 
 void spectrumMouseAction(GLwDrawingAreaCallbackStruct *data, int state)
 {
+  const GLfloat black[4] = {0.0, 0.0, 0.0, 0.0};
   static struct PushButtonRow button[] = {{"continue", GetMessageBoxButton, (XtPointer)0, NULL}};
-  char *word;
+  char line[MAXLENLINE], *word;
 
   if (data->event->xbutton.button == Button1 && state == TRUE)
   {
@@ -293,10 +320,15 @@ void spectrumMouseAction(GLwDrawingAreaCallbackStruct *data, int state)
           if (animate == DISTORT) distortGeometry(amplitude);
           drawMolecule((Widget)0, (caddr_t)0, data);
         }
+        word=getStringResource(topShell, "wavenumberTitle");
+        sprintf(line, word, normal_modes[mode].sym, wnScale*normal_modes[mode].wavenumber);
+	  normalModeAnnotation=makeAnnotation(normalModeAnnotation, CENTERED, 0.0,
+							  1.0, stdcol[BLACK], black, 0, line);
       }
       else
       {
         if (animate == ANIMATE) setAnimation(FALSE);
+	  deleteAnnotation(&normalModeAnnotation);
         restoreGeometry();
         drawMolecule((Widget)0, (caddr_t)0, data);
       }
@@ -305,7 +337,7 @@ void spectrumMouseAction(GLwDrawingAreaCallbackStruct *data, int state)
   else if (data->event->xbutton.button == Button2 && state == TRUE)
     initZoom(data);
   else if (data->event->xbutton.button == Button2 && state == FALSE)
-    endZoom(data->width, data->height, data->event->xbutton.x, data->event->xbutton.y);
+    endZoom(data->height, data->event->xbutton.x, data->event->xbutton.y);
 }
 
 void getImaginaryMode(Widget widget, caddr_t which, caddr_t data)
@@ -319,22 +351,20 @@ void getImaginaryMode(Widget widget, caddr_t which, caddr_t data)
   }
 }
 
-void historyMouseAction(unsigned int button, int state, int x, int y,
-                        Dimension width, Dimension height)
+void historyMouseAction(unsigned int button, int state, int x, int y)
 {
   double xpix, ypix;
 
   if (button == Button1 && state == TRUE)
   {
-    pixwor(width, height, windows[HISTORY].left, windows[HISTORY].right,
-           windows[HISTORY].bottom, windows[HISTORY].top, &xpix, &ypix);
-    cycle=(double)x*xpix+1.0e0;
+    pixelToWorld(HISTORY, &xpix, &ypix);
+    cycle=(double)x*xpix+1.0;
     redraw(HISTORY);
-    if (cycle < 1.0e0) cycle=1.0e0;
+    if (cycle < 1.0) cycle=1.0;
     if (cycle > (double)nhist) cycle=(double)nhist;
   }
   if (button == Button1 && state == FALSE)
-    cycle=ceil(cycle-0.5e0);
+    cycle=ceil(cycle-0.5);
 
   setGeometry(TRUE);
   redraw(VIEWER);
@@ -356,17 +386,18 @@ void MODiagramMouseAction(GLwDrawingAreaCallbackStruct *data, int state)
       ringBell();
       if (imo != imoSave && grid != NULL)
         fremem((void **)&grid);
-      if (orbitals[imo].occupation == 0.0 && needMoloch) setMenuItem(VIEWER, 6, False);
-      else if (basisset != NULL)                         setMenuItem(VIEWER, 6, True);
+      if (orbitals[imo].occupation == 0.0 && needMoloch) setMenuItem(VIEWER, 7, False);
+      else if (basisset != NULL)                         setMenuItem(VIEWER, 7, True);
       setMenuItem(MO, 1, True);
     }
     else
       setMenuItem(MO, 1, False);
+    annotateWavefunction();
   }
   else if (data->event->xbutton.button == Button2 && state == TRUE)
     initZoom(data);
   else if (data->event->xbutton.button == Button2 && state == FALSE)
-    endZoom(data->width, data->height, data->event->xbutton.x, data->event->xbutton.y);
+    endZoom(data->height, data->event->xbutton.x, data->event->xbutton.y);
 }
 
 GLint doPicking(int x, int y, void draw())
@@ -419,11 +450,13 @@ GLint doPicking(int x, int y, void draw())
 
 void setGeometry(int doConnectivity)
 {
+  const GLfloat black[4] = {0.0, 0.0, 0.0, 0.0};
   double top, bottom, right, left, force;
+  char line[MAXLENLINE], *word;
   register int i, j;
 
   naSaved=na;
-  j=((int)ceil(cycle-0.5e0)-1);
+  j=((int)ceil(cycle-0.5)-1);
 /*if (j > 0) na=optimization[j].natoms; */
   i=optimization[j].coords;
   for (j=0; j<na; j++, i++)
@@ -432,12 +465,19 @@ void setGeometry(int doConnectivity)
     atoms[j].y=history[i].y;
     atoms[j].z=history[i].z;
   }
+  word=getStringResource(topShell, "historyTitle");
+  sprintf(line, word, (int)cycle, optimization[(int)cycle-1].energy,
+	    optimization[(int)cycle-1].gnorm);
+  historyAnnotation=makeAnnotation(historyAnnotation, CENTERED, 0.0, 1.0,
+					     stdcol[BLACK], black, 0, line);
+
   if (!doConnectivity) return;
   top=windows[VIEWER].top;
   bottom=windows[VIEWER].bottom;
   right=windows[VIEWER].right;
   left=windows[VIEWER].left;
   force=forceScale;
+  fremem((void **)&bonds);
   makeConnectivity(existsUnitCell);
   windows[VIEWER].top=top;
   windows[VIEWER].bottom=bottom;
