@@ -4,15 +4,17 @@
 *                                                                              *
 *                              M O U S E C B . C                               *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: mousecb.c,v 1.1 1996/12/10 18:42:34 jrh Exp $
+* $Id: mousecb.c,v 1.2 1998/01/26 00:48:52 jrh Exp jrh $
 * $Log: mousecb.c,v $
- * Revision 1.1  1996/12/10  18:42:34  jrh
- * Initial revision
- *
+* Revision 1.2  1998/01/26 00:48:52  jrh
+* Release 2.1
+*
+* Revision 1.1  1996/12/10  18:42:34  jrh
+* Initial revision
 *
 */
 #include<X11/Intrinsic.h>
@@ -40,7 +42,7 @@ extern void drawHistory(Widget, caddr_t, GLwDrawingAreaCallbackStruct *);
 extern void drawMODiagram(Widget, caddr_t, GLwDrawingAreaCallbackStruct *);
 extern void viewerMouseAction(XEvent *, int);
 extern void spectrumMouseAction(GLwDrawingAreaCallbackStruct *, int);
-extern void historyMouseAction(unsigned int, int, int, int, Dimension, Dimension);
+extern void historyMouseAction(unsigned int, int, int, int);
 extern void MODiagramMouseAction(GLwDrawingAreaCallbackStruct *, int);
 extern void viewerKeyAction(KeySym);
 extern void spectrumKeyAction(KeySym);
@@ -53,11 +55,10 @@ extern void zoomIn(Widget, GLwDrawingAreaCallbackStruct *);
 
 extern struct WINDOW windows[];
 extern struct ORBITAL *orbitals;
-extern float xrot, yrot, zrot;
 extern int lines, label, showForces;
 extern int rotateXY, rotateZ, imo;
 extern int showMenu, primitive, selectAtom, iwavef;
-extern int picking, swapBuffers, movelight, simplify;
+extern int picking, swapBuffers, moveItem, simplify;
 extern Widget bfMenu;
 
 void processInput(Widget w, caddr_t client_data,  GLwDrawingAreaCallbackStruct *data)
@@ -73,7 +74,8 @@ void processInput(Widget w, caddr_t client_data,  GLwDrawingAreaCallbackStruct *
     {
       case ButtonPress:   if (w == windows[VIEWER].widget)
                           {
-                            if (data->event->xbutton.button != Button3 && simplify && movelight == (-1))
+                            if (data->event->xbutton.button != Button3 && simplify
+                                && (moveItem & 0x7fffffff) < LIGHTNO0)
                             {
                               primitiveSave=primitive;
                               primitive=GLU_LINE;
@@ -84,14 +86,13 @@ void processInput(Widget w, caddr_t client_data,  GLwDrawingAreaCallbackStruct *
                             spectrumMouseAction(data, TRUE);
                           else if (w == windows[HISTORY].widget)
                             historyMouseAction(data->event->xbutton.button, TRUE,
-                                               data->event->xbutton.x, data->event->xbutton.y,
-                                               data->width, data->height);
+                                               data->event->xbutton.x, data->event->xbutton.y);
                           else if (w == windows[MO].widget)
                             MODiagramMouseAction(data, TRUE);
                           break;
       case ButtonRelease: if (w == windows[VIEWER].widget)
                           {
-                            if (data->event->xbutton.button != Button3 && simplify && movelight == (-1))
+                            if (data->event->xbutton.button != Button3 && simplify && moveItem < LIGHTNO0)
                               primitive=primitiveSave;
                             viewerMouseAction(data->event, FALSE);
                             rotateXY=rotateZ=0;
@@ -101,8 +102,7 @@ void processInput(Widget w, caddr_t client_data,  GLwDrawingAreaCallbackStruct *
                             spectrumMouseAction(data, FALSE);
                           else if (w == windows[HISTORY].widget)
                             historyMouseAction(data->event->xbutton.button, FALSE,
-                                               data->event->xbutton.x, data->event->xbutton.y,
-                                               data->width, data->height);
+                                               data->event->xbutton.x, data->event->xbutton.y);
                           else if (w == windows[MO].widget)
                             MODiagramMouseAction(data, FALSE);
                           break;
@@ -128,8 +128,7 @@ void processInput(Widget w, caddr_t client_data,  GLwDrawingAreaCallbackStruct *
                             else if (w == windows[HISTORY].widget)
                             {
                               historyMouseAction(Button1, TRUE, data->event->xmotion.x,
-                                                 data->event->xmotion.y,
-                                                 data->width, data->height);
+                                                 data->event->xmotion.y);
                             }
                           }
                           else if (data->event->xmotion.state & Button2Mask)
@@ -192,9 +191,15 @@ void setModel(Widget w, caddr_t model, GLwDrawingAreaCallbackStruct *data)
   windows[VIEWER].mode=(int)model;
   setMenuItem(VIEWER, (int)model-WIREMODEL, False);
   if ((int)model != WIREMODEL)
-    setMenuItem(VIEWER, 19, True);
+  {
+    setMenuItem(VIEWER, 6, False);
+    setMenuItem(VIEWER, 22, True);
+  }
   else
-    setMenuItem(VIEWER, 19, False);
+  {
+    setMenuItem(VIEWER, 6, True);
+    setMenuItem(VIEWER, 22, False);
+  }
   drawMolecule((Widget)0, (caddr_t)0, (GLwDrawingAreaCallbackStruct *)0);
 }
 
