@@ -4,12 +4,15 @@
 *                                                                              *
 *                            R E A D G A U S S . C                             *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: readgauss.c,v 1.1 1996/12/10 18:43:35 jrh Exp $
+* $Id: readgauss.c,v 1.2 1998/01/26 00:49:14 jrh Exp jrh $
 * $Log: readgauss.c,v $
+* Revision 1.2  1998/01/26 00:49:14  jrh
+* Release 2.1
+*
 * Revision 1.1  1996/12/10  18:43:35  jrh
 * Initial revision
 *
@@ -41,11 +44,14 @@
 #define AMOS "alpha mos:"
 #define BMOS "beta  mos:"
 #define AXES "***** axes restored to original set *****"
+#define SYMMETRIES 1
+#define ALPHAMOS   2
 
 void readBasis(FILE *, int);
-void readMOs(FILE *);
+void readMOs(int, FILE *);
 void readFrq(FILE *, int, int);
-void readOpg(FILE *, int, int, char *);
+int  readOpg(FILE *, int, int, char *);
+void printCoord(double **, char **, int);
 double reade(char *, char *, int *);
 int msub(char *, int, char *);
 void makeLower(char *);
@@ -63,7 +69,7 @@ void main(int argc, char **argv)
   char emark[2][7] = {"0@  ~0", "0@  ~0"}, line[MAXLENLINE], *word;
   char *symbol;
   int freq=FALSE, gfprn=FALSE, opt=FALSE;
-  int nemark=0, nout, na, ncent, nuc, mna=50;
+  int nemark=0, nout, na, ncent, nuc, mna=50, cycle=0;
 /*int etex; */
   register double x, y, z;
   register int i; /* mak;*/
@@ -183,7 +189,7 @@ void main(int argc, char **argv)
   fgets(line, MAXLENLINE, file);
   na=0;
   ncent=0;
-  printf("$coord 1.0\n");
+  if (!opt) printf("$coord 1.0\n");
   symbol=(char *)getmem(mna, 3*sizeof(char));
   while (fgets(line, MAXLENLINE, file) != NULL)
   {
@@ -200,7 +206,7 @@ void main(int argc, char **argv)
 	symbol[3*na]=pse[nuc][0];
 	symbol[3*na+1]=pse[nuc][1];
 	symbol[3*na+2]='\0';
-	printf("%22.14f%22.14f%22.14f  %s\n", x, y, z, &symbol[3*na]);
+	if (!opt) printf("%22.14f%22.14f%22.14f  %s\n", x, y, z, &symbol[3*na]);
 	if (++na >= mna)
 	{
 	  mna+=50;
@@ -209,6 +215,10 @@ void main(int argc, char **argv)
     }
   }
 
+/* Load optimization history */
+
+  if (opt) cycle=readOpg(file, na, nemark, &emark[0][0]);
+
 /* Load basis functions */
 
   if (gfprn)
@@ -234,7 +244,7 @@ void main(int argc, char **argv)
 
 /* Load orbital energies and symmetries */
 
-  readMOs(file);
+  readMOs(cycle, file);
   if (na <= 1)
   {
     freq=FALSE;
@@ -245,10 +255,6 @@ void main(int argc, char **argv)
 
   if (freq) readFrq(file, na, nout);
 
-/* Load optimization history */
-
-  if (opt) readOpg(file, na, nemark, &emark[0][0]);
-
   printf("$end\n");
   fclose(file);
 }
@@ -301,7 +307,7 @@ void readBasis(FILE *file, int na)
       }
 	else
 	{
-        strncpy(shell, word, 2);
+        strcpy(shell, word);
         scalefactor=atof(strtok(NULL, " \t"));
 	}
     }
@@ -328,11 +334,11 @@ void readBasis(FILE *file, int na)
     }
     if (shell[0] == '?')
     {
-      if (scoeff[0] != 0.0) shell[0]='S';
-      if (pcoeff[0] != 0.0) shell[0]='P';
+      if (scoeff[0] != 0.0) strcpy(shell, "S");
+      if (pcoeff[0] != 0.0) strcpy(shell, "P");
 	if (scoeff[0] != 0.0 && pcoeff[0] != 0.0) strcpy(shell, "SP");
-      if (dcoeff[0] != 0.0) shell[0]='D';
-      if (fcoeff[0] != 0.0) shell[0]='F';
+      if (dcoeff[0] != 0.0) strcpy(shell, "D");
+      if (fcoeff[0] != 0.0) strcpy(shell, "F");
     }
     for (word=&shell[0]; *word; word++)
     {
@@ -373,10 +379,11 @@ void readBasis(FILE *file, int na)
   fremem((void **)&fcoeff);
 }
 
-void readMOs(FILE *file)
+void readMOs(int cycle, FILE *file)
 {
   double *eorb, *cmo;
-  int ia=0, ie, mno=20, mnc=200, norbital, offset, occ=TRUE, nocc=0, found;
+  int ia=0, ie, mno=20, mnc=200, norbital=0, offset, occ=TRUE, nocc=0;
+  int info, found;
   char line[MAXLENLINE], number[8], *word, *begin, *symorb, *closed;
   register int i, j, k;
 
@@ -385,65 +392,99 @@ void readMOs(FILE *file)
   {
     if (fgets(line, MAXLENLINE, file) == NULL) return;
     makeLower(line);
-    if (strstr(line, OSYM) && !strstr(line, "in")) break;
+    if (strstr(line, OSYM) && !strstr(line, "in"))
+    {
+      info=SYMMETRIES;
+      break;
+    }
+    if (strstr(line, AMOS))
+    {
+      info=ALPHAMOS;
+      break;
+    }
   }
+
   symorb=(char *)getmem(mno, 4*sizeof(char));
   closed=(char *)getmem(20, MAXLENLINE*sizeof(char));
-  i=0;
-
-  while (fgets(line, MAXLENLINE, file))
+  if (info == SYMMETRIES)
   {
-    makeLower(line);
-    if ((word=strrchr(line, '\n')) != NULL) *word='\0';
-    if (strstr(line, "the electronic state") || strstr(line, " alpha deviation from unit magnitude")) break;
-    if (strstr(line, "occupied") || strstr(line, "virtual"))
+    i=0;
+
+    while (fgets(line, MAXLENLINE, file))
     {
-	if (strstr(line, "virtual")) occ=FALSE;
-	strtok(line, " \t");
-	begin=NULL;
-    }
-    else
-	begin=line;
-    while ((word=strtok(begin, " \t")) != NULL)
-    {
-	begin=NULL;
-      strncpy(&symorb[4*i], word+1, 4);
-      if ((word=strrchr(&symorb[4*i], ')')) != NULL) *word='\0';
-	if (occ)
-	{
-	  found=FALSE;
-	  for (j=0; j<nocc; j++)
+      makeLower(line);
+      if ((word=strrchr(line, '\n')) != NULL) *word='\0';
+      if (strstr(line, "the electronic state") || strstr(line, " alpha deviation from unit magnitude")) break;
+      if (strstr(line, "occupied") || strstr(line, "virtual"))
+      {
+    	  if (strstr(line, "virtual")) occ=FALSE;
+  	  strtok(line, " \t");
+	  begin=NULL;
+      }
+      else
+  	  begin=line;
+      while ((word=strtok(begin, " \t")) != NULL)
+      {
+	  begin=NULL;
+        strncpy(&symorb[4*i], word+1, 4);
+        if ((word=strrchr(&symorb[4*i], ')')) != NULL) *word='\0';
+	  if (occ)
 	  {
-	    if (!strncmp(&symorb[4*i], &closed[MAXLENLINE*j], strlen(&symorb[4*i])))
+	    found=FALSE;
+	    for (j=0; j<nocc; j++)
 	    {
-		sprintf(number, ",%d", i+1);
-		strcat(&closed[MAXLENLINE*j], number);
-		found=TRUE;
-            break;
+	      if (!strncmp(&symorb[4*i], &closed[MAXLENLINE*j], strlen(&symorb[4*i])))
+	      {
+	  	  sprintf(number, ",%d", i+1);
+		  strcat(&closed[MAXLENLINE*j], number);
+		  found=TRUE;
+              break;
+	      }
+	    }
+	    if (!found)
+	    {
+	      sprintf(&closed[MAXLENLINE*nocc], "%s %d", &symorb[4*i], i+1);
+	      nocc++;
 	    }
 	  }
-	  if (!found)
-	  {
-	    sprintf(&closed[MAXLENLINE*nocc], "%s %d", &symorb[4*i], i+1);
-	    nocc++;
-	  }
-	}
-      i++;
-      if (i >= mno)
-      {
-    	  mno+=20;
-	  symorb=(char *)expmem((void *)symorb, mno, 4*sizeof(char));
+        i++;
+        if (i >= mno)
+        {
+    	    mno+=20;
+	    symorb=(char *)expmem((void *)symorb, mno, 4*sizeof(char));
+        }
       }
     }
+    norbital=i;
+  }
+  else
+  {
+    rewind(file);
+    do
+    {
+      if (fgets(line, MAXLENLINE, file) == NULL) return;
+      makeLower(line);
+    } while (!strstr(line, "symmetry adapted basis"));
+    while (strstr(line, "symmetry adapted basis"))
+    {
+      strtok(line, " \t");
+      strtok(NULL, " \t");
+      norbital+=atof(strtok(NULL, " \t"));
+      fgets(line, MAXLENLINE, file);
+      makeLower(line);
+    }
+    symorb=(char *)expmem((void *)symorb, norbital, 4*sizeof(char));
+    strcpy(closed, "A1 1-");
   }
-  norbital=i;
 
   rewind(file);
   do
   {
     if (fgets(line, MAXLENLINE, file) == NULL) return;
     makeLower(line);
-  } while (!strstr(line, AMOS));
+    word=strstr(line, AMOS);
+    if (word) cycle--;
+  } while (!word || cycle > 0);
 
   printf("$scfmo gaussian\n");
   eorb=(double *)getmem(norbital, sizeof(double));
@@ -459,7 +500,19 @@ void readMOs(FILE *file)
     strtok(line, " \t");
     strtok(NULL, " \t");
     for (i=ia; i<ia+ie; i++)
+    {
       eorb[i]=atof(strtok(NULL, " \t"));
+      if (info == ALPHAMOS)
+      {
+        strcpy(&symorb[4*i], "A1");
+        if (eorb[i] > 0.0 && nocc == 0)
+        {
+          sprintf(number, "%d", i+1);
+          strcat(closed, number);
+          nocc++;
+        }
+      }
+    }
 
     k=0;
     for (j=0; j<norbital; j++)
@@ -498,9 +551,9 @@ void readMOs(FILE *file)
   for (i=0; i<nocc; i++)
     printf(" %s\n", &closed[MAXLENLINE*i]);
   fremem((void **)&symorb);
+  fremem((void **)&closed);
   fremem((void **)&eorb);
   fremem((void **)&cmo);
-  fremem((void **)&closed);
 }
 
 void readFrq(FILE *file, int na, int nout)
@@ -509,7 +562,7 @@ void readFrq(FILE *file, int na, int nout)
   double *freq, *ir, *raman, *nmo;
   int ncart=3*na-nout, nfrq=0;
   char line[MAXLENLINE], *symmod, *word;
-  register int i, j, end;
+  register int i, j, k, end;
 
   rewind(file);
   do
@@ -532,9 +585,13 @@ void readFrq(FILE *file, int na, int nout)
     fgets(line, MAXLENLINE, file);
     fgets(line, MAXLENLINE, file);
     strcpy(&symmod[4*nfrq], strtok(line, " \t\n"));
-    end=nfrq+5 < ncart ? nfrq+5 : ncart;
-    for (i=nfrq+1; i<end; i++)
-      strcpy(&symmod[4*i], strtok(NULL, " \t\n"));
+    i=nfrq+1;
+    while ((word=strtok(NULL, " \t\n")) != NULL)
+    {
+      strcpy(&symmod[4*i], word);
+	i++;
+    }
+    end=i;
 
     fgets(line, MAXLENLINE, file);
     strtok(line, " \t");
@@ -560,16 +617,35 @@ void readFrq(FILE *file, int na, int nout)
 
     fgets(line, MAXLENLINE, file);
     fgets(line, MAXLENLINE, file);
-    for (i=0; i<ncart+nout; i++)
+    makeLower(line);
+    if (strstr(line, "x      y      z"))
     {
-	fgets(line, MAXLENLINE, file);
-	strtok(line, " \t");
-	strtok(NULL, " \t");
-	strtok(NULL, " \t");
-	for (j=nfrq; j<end; j++)
-	  nmo[i*ncart+j]=atof(strtok(NULL, " \t"));
+	for (i=0; i<na; i++)
+	{
+  	  fgets(line, MAXLENLINE, file);
+	  strtok(line, " \t");
+	  strtok(NULL, " \t");
+	  for (j=nfrq; j<end; j++)
+	  {
+	    for (k=0; k<3; k++)
+            nmo[(3*i+k)*ncart+j]=atof(strtok(NULL, " \t"));
+	  }
+	}
+	nfrq+=3;
+    }
+    else
+    {
+      for (i=0; i<ncart+nout; i++)
+      {
+  	  fgets(line, MAXLENLINE, file);
+	  strtok(line, " \t");
+	  strtok(NULL, " \t");
+	  strtok(NULL, " \t");
+	  for (j=nfrq; j<end; j++)
+	    nmo[i*ncart+j]=atof(strtok(NULL, " \t"));
+      }
+      nfrq+=5;
     }
-    nfrq+=5;
   }
   printf("$vibrational spectrum\n");
   word=symmod;
@@ -603,7 +679,7 @@ void readFrq(FILE *file, int na, int nout)
   fremem((void **)&nmo);
 }
 
-void readOpg(FILE *file, int na, int nemark, char *emark)
+int readOpg(FILE *file, int na, int nemark, char *emark)
 {
   double *x, *y, *z, *gx, *gy, *gz;
   double energy, gnorm;
@@ -626,9 +702,8 @@ void readOpg(FILE *file, int na, int nemark, char *emark)
     {
       if (fgets(line, MAXLENLINE, file) == NULL)
 	{
-        fremem((void **)&x);
-        fremem((void **)&symbol);
-	  return;
+        printCoord(&x, &symbol, na);
+	  return(cycle);
       }
 	makeLower(line);
     } while (!strstr(line, STAOR));
@@ -642,9 +717,8 @@ void readOpg(FILE *file, int na, int nemark, char *emark)
     {
       if (fgets(line, MAXLENLINE, file) == NULL)
 	{
-        fremem((void **)&x);
-        fremem((void **)&symbol);
-	  return;
+        printCoord(&x, &symbol, na);
+	  return(cycle);
       }
       if (strstr(line, TABMAR)) break;
       strtok(line, " \t");
@@ -669,9 +743,8 @@ void readOpg(FILE *file, int na, int nemark, char *emark)
     {
       if (fgets(line, MAXLENLINE, file) == NULL)
 	{
-        fremem((void **)&x);
-        fremem((void **)&symbol);
-	  return;
+        printCoord(&x, &symbol, na);
+	  return(cycle);
 	}
 	makeLower(line);
     } while (!strstr(line, AXES));
@@ -684,9 +757,8 @@ void readOpg(FILE *file, int na, int nemark, char *emark)
     {
       if (fgets(line, MAXLENLINE, file) == NULL)
 	{
-        fremem((void **)&x);
-        fremem((void **)&symbol);
-	  return;
+        printCoord(&x, &symbol, na);
+	  return(cycle);
 	}
       if (strstr(line, TABMAR)) break;
       strtok(line, " \t");
@@ -708,6 +780,26 @@ void readOpg(FILE *file, int na, int nemark, char *emark)
     for (i=0; i<na; i++)
       printf("%22.14e%22.14e%22.14e\n", gx[i], gy[i], gz[i]);
   }
+  /* We should never reach this point */
+  return(cycle);
+}
+
+void printCoord(double **coord, char **symbol, int na)
+{
+  double *x, *y, *z;
+  char *s;
+  register int i;
+
+  x=*coord;
+  y=x+na;
+  z=y+na;
+  s=*symbol;
+  printf("$coord 1.0\n");
+  for (i=0; i<na; i++)
+    printf("%22.14f%22.14f%22.14f  %s\n", x[i], y[i], z[i], &s[3*i]);
+
+  fremem((void **)coord);
+  fremem((void **)symbol);
 }
 
 double reade(char *line, char *emark, int *done)
