@@ -4,16 +4,18 @@
 *                                                                              *
 *                              V I E W M O L . C                               *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 1996                   *
+*                 Copyright (c) Joerg-R. Hill, December 1997                   *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: viewmol.c,v 1.1 1996/12/10 18:44:18 jrh Exp $
+* $Id: viewmol.c,v 1.2 1998/01/26 00:49:40 jrh Exp jrh $
 * $Log: viewmol.c,v $
+* Revision 1.2  1998/01/26 00:49:40  jrh
+* Release 2.1
+*
 * Revision 1.1  1996/12/10  18:44:18  jrh
 * Initial revision
 *
-*
 */
 #include<signal.h>
 #include<stdio.h>
@@ -40,27 +42,11 @@ void getRotation(int, int);
 void setDrawingDevice(int);
 void ende(Widget, caddr_t, XmAnyCallbackStruct *);
 
-extern void hpglBegin(GLenum), hpglEnd(void), hpglVertex3d(double, double, double);
-extern void hpglNormal3d(double, double, double);
-extern void hpglVertex2d(double, double), hpglColor4fv(const GLfloat *);
-extern void plotString(char *, double, double, double, GLuint);
-extern void hpglLineStipple(GLint, GLushort), hpglDisable(GLenum);
-extern void hpglLineWidth(GLfloat);
-extern void hpglClearColor(GLclampf, GLclampf, GLclampf, GLclampf);
-extern void hpglSphere(GLUquadricObj *, GLdouble, GLint, GLint);
-extern void hpglCylinder(GLUquadricObj *qobj, GLdouble, GLdouble, GLdouble, GLint, GLint);
-extern void hpglOrtho(GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble);
-extern void hpglPopMatrix(void);
-extern void PostscriptBegin(GLenum), PostscriptEnd(void), PostscriptVertex3d(double, double, double);
-extern void PostscriptNormal3d(double, double, double);
-extern void PostscriptVertex2d(double, double), PostscriptColor4fv(const GLfloat *);
-extern void PostscriptString(char *, double, double, double, GLuint);
-extern void printString(char *, double, double, double, GLuint);
-extern void PostscriptLineStipple(GLint, GLushort), PostscriptDisable(GLenum);
-extern void PostscriptLineWidth(GLfloat);
-extern void PostscriptClearColor(GLclampf, GLclampf, GLclampf, GLclampf);
-extern void PostscriptOrtho(GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble);
-extern void PostscriptPopMatrix(void);
+extern void printString(char *, double, double, double, double, GLuint);
+extern void feedbackString(char *, double, double, double, double, GLuint);
+extern void feedbackLineStipple(GLint, GLushort);
+extern void feedbackLineWidth(GLfloat);
+extern void feedbackDisable(GLenum);
 extern void raytracerBegin(GLenum), raytracerEnd(void), raytracerVertex3d(double, double, double);
 extern void raytracerNormal3d(double, double, double);
 extern void raytracerColor4fv(const GLfloat *);
@@ -69,26 +55,26 @@ extern void raytracerSphere(GLUquadricObj *, GLdouble, GLint, GLint);
 extern void raytracerCylinder(GLUquadricObj *qobj, GLdouble, GLdouble, GLdouble, GLint, GLint);
 extern void GetMessageBoxButton(Widget, XtPointer, caddr_t);
 extern char *getStringResource(Widget, char *);
-extern void pixwor(Dimension, Dimension, GLdouble, GLdouble, GLdouble, GLdouble, double *, double *);
-extern int messgb(Widget, int, char *, struct PushButtonRow *, int);
+extern void pixelToWorld(int, double *, double *);
+extern int  messgb(Widget, int, char *, struct PushButtonRow *, int);
 extern void loadColorMap(void);
-extern int getrc(void);
+extern int  getrc(void);
 extern void showTitle(Dimension, Dimension);
-extern int initViewer(int, char **, Widget);
+extern int  initViewer(int, char **, Widget);
 extern void redraw(int);
 extern void fremem(void **);
+extern void scaleAnnotation(float);
 
 void (*drawBegin)(GLenum), (*drawEnd)(void), (*drawVertex3d)(double, double, double);
 void (*drawVertex2d)(double, double), (*drawNormal3d)(double, double, double);
 void (*drawColor4fv)(const GLfloat *);
-void (*drawString)(char *, double, double, double, GLuint);
-void (*drawLineStipple)(GLint, GLushort), (*drawDisable)(GLenum);
-void (*drawLineWidth)(GLfloat);
 void (*drawClearColor)(GLclampf, GLclampf, GLclampf, GLclampf);
 void (*drawSphere)(GLUquadricObj *, GLdouble, GLint, GLint);
 void (*drawCylinder)(GLUquadricObj *qobj, GLdouble, GLdouble, GLdouble, GLint, GLint);
-void (*drawOrtho)(GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble);
-void (*drawPopMatrix)(void);
+void (*drawString)(char *, double, double, double, double, GLuint);
+void (*drawLineStipple)(GLint, GLushort);
+void (*drawLineWidth)(GLfloat);
+void (*drawDisable)(GLenum);
 
 struct ATOM *atoms;
 struct BOND *bonds;
@@ -96,45 +82,50 @@ struct ELEMENT *elements;
 struct NORMAL_MODE *normal_modes=NULL;
 struct WINDOW windows[4];
 struct OPTION *options=NULL;
+struct OPTION *output=NULL;
 struct COORDS *history=NULL;
 struct INTERNAL *internals=NULL;
 struct OPTIMIZATION *optimization=NULL;
 struct ORBITAL *orbitals=NULL;
 struct BASISSET *basisset=NULL;
 struct GRIDOBJECT *gridObjects=NULL;
+struct ANNOTATION *annotation=NULL;
 Pixel stdcol[9], historyColor;
 char title[MAXLENLINE], webbrowser[MAXLENLINE]={""};
-int na, nb, ne, nmodes, nopt=0, nhist=0, ninternal=0, ngridobjects=0;
+int na, nb, ne, nmodes, nopt=0, noutput=0, nhist=0, ninternal=0, ngridobjects=0;
 double *cnm=NULL, *coord, *exponents=NULL, *grid=NULL;
-double radfac, bndfac=1.0e0, amplitude, forceScale, wnScale=1.0;
+double radfac, bndfac=1.0, amplitude, forceScale, wnScale=1.0;
 double transx, transy, transz;
-double tinert[3][3], tmat[4][4], prmat[4][4];
-double paperWidth=0.0, paperHeight=0.0, sphereres=10.;
-float xrot=0.0, yrot=0.0, zrot=0.0;
-float xmov=0.0, ymov=0.0;
+double tinert[3][3], tmat[4][4];
+double paperWidth=0.0, paperHeight=0.0, sphereres=10., lineWidth=0.0;
+double temp, weight=0.0;
+double emax, emin, cycle;
+double e1, e2, denres=(1.);
+double level=0.05, gridres=(-1.);
+float xrot[4]={0.0, 0.0, 0.0, 0.0}, yrot[4]={0.0, 0.0, 0.0, 0.0};
+float zrot[4]={0.0, 0.0, 0.0, 0.0};
+float xmov[2]={0.0, 0.0}, ymov[2]={0.0, 0.0}, zmov[2]={0.0, 0.0};
 float historyColor_rgb[4] = {0.0, 0.0, 1.0, 0.0};
 GLfloat light0p[4] = {0.0, 0.0, 1.0, 0.0};
 GLfloat light1p[4] = {0.0, 1.0, 0.0, 0.0};
 int rotateXY=FALSE, rotateZ=FALSE;
-int primitive, animate=ANIMATE, lights=3, movelight=(-1);
+int primitive, animate=ANIMATE, lights=3, moveItem=0, projectionMode=ORTHO;
 int lines=TRUE, mode=(-1), showForces=FALSE, setins=FALSE;
 int existsUnitCell=FALSE, showUnitCell=TRUE;
 int showInertia=FALSE, selectAtom=FALSE, simplify=TRUE;
-int rgbMode, picking=FALSE, swapBuffers=TRUE;
-int drawingDevice;
-double temp, weight=0.0e0;
-int label, imag=0;
-int debug;
-double emax, emin, cycle;
-int nhist;
-double e1, e2, denres=(1.);
+int rgbMode, picking=FALSE, swapBuffers=TRUE, shadows=FALSE;
+int drawingDevice, bondType=CONJUGATION;
+int label, imag=0, outputType=0;
+int debug, annotate=FALSE;
+int nhist, nAnnotations=0, normalModeAnnotation=(-1), selectAtomAnnotation=(-1);
+int historyAnnotation=(-1), setinsAnnotation=(-1), wavefunctionAnnotation[4]={-1, -1, -1, -1};
 int nbasfu, unit, imo=(-1), imosav=(-1), ibasfu=(-1), bfatom;
 char *formBondLength, *formBondAngle, *formTorsionAngle;
+char *text, *textPointer;
 XtAppContext app;
 Widget topShell, fileBox, bfMenu;
 int showMenu=TRUE;
 int iwavef=ALL_OFF, interp=IP_LINEAR;
-double level=0.05, gridres=(-1.);
 int needMoloch=FALSE, gaussian=FALSE;
 char moloch[MAXLENLINE];
 pid_t manual_pid=0, raypid=0;
@@ -153,6 +144,7 @@ int main(int argc, char** argv)
                   GLX_DEPTH_SIZE, 1,
                   GLX_DOUBLEBUFFER,
                   GLX_RGBA,
+                  GLX_STENCIL_SIZE, 2,
                   None};
   static struct PushButtonRow buttons[] = {{"exit", GetMessageBoxButton, (XtPointer)0, NULL}};
   char *line;
@@ -183,19 +175,23 @@ int main(int argc, char** argv)
   XtRealizeWidget(topShell);
 
   XtVaGetValues(viewer, XmNwidth, &width, XmNheight, &height, NULL);
-#ifndef MESA
-  if (rgbMode)
-    colormap=XCreateColormap(display, XtWindow(viewer), vi->visual, AllocNone);
-  else
-#endif
-    colormap=DefaultColormap(display, DefaultScreen(display));
-
   windows[VIEWER].widget=XtVaCreateManagedWidget("canvas", glwMDrawingAreaWidgetClass, viewer,
                                                  GLwNvisualInfo, vi,
                                                  XmNwidth, width,
                                                  XmNheight, height,
-                                                 XmNcolormap, colormap,
                                                  NULL);
+#ifndef LINUX
+  if (!rgbMode)
+  {
+#endif
+    if (vi->visual == DefaultVisual(display, DefaultScreen(display)))
+      colormap=DefaultColormap(display, DefaultScreen(display));
+    else
+      colormap=XCreateColormap(display, XtWindow(viewer), vi->visual, AllocNone);
+    XtVaSetValues(windows[VIEWER].widget, XmNcolormap, colormap, NULL);
+#ifndef LINUX
+  }
+#endif
   windows[VIEWER].context=glXCreateContext(XtDisplay(windows[VIEWER].widget), vi, 0, GL_FALSE);
   GLwDrawingAreaMakeCurrent(windows[VIEWER].widget, windows[VIEWER].context);
 
@@ -256,12 +252,12 @@ void getRotation(int x, int y)
 
   if (rotateXY == 1)
   {
-    xrot+=(float)(y-yold);
-    yrot+=(float)(x-xold);
+    xrot[moveItem]+=(float)(y-yold);
+    yrot[moveItem]+=(float)(x-xold);
   }
   else if (rotateZ == 1)
-    zrot+=(float)(x-xold);
- 
+    zrot[moveItem]+=(float)(x-xold);
+
   if (rotateXY == -1)
     rotateXY=1;
   else if (rotateZ == -1)
@@ -273,17 +269,22 @@ void getRotation(int x, int y)
 
 void getTranslation(int x, int y)
 {
-  Dimension width, height;
   static int xold, yold;
   double xpix, ypix;
 
   if (rotateXY == 1)
   {
-    XtVaGetValues(windows[VIEWER].widget, XtNwidth, &width, XtNheight, &height, NULL);
-    pixwor(width, height, windows[VIEWER].left, windows[VIEWER].right, windows[VIEWER].bottom,
-         windows[VIEWER].top, &xpix, &ypix);
-    xmov+=(float)(x-xold)*xpix;
-    ymov+=(float)(yold-y)*ypix;
+    pixelToWorld(VIEWER, &xpix, &ypix);
+    if (moveItem & 0x80000000)
+    {
+	annotation[moveItem & 0x7fffffff].x+=(double)(x-xold)*xpix;
+	annotation[moveItem & 0x7fffffff].y+=(double)(yold-y)*ypix;
+    }
+    else
+    {
+      xmov[moveItem]+=(float)(x-xold)*xpix;
+      ymov[moveItem]+=(float)(yold-y)*ypix;
+    }
   }
   if (rotateXY == -1) rotateXY=1;
 
@@ -298,11 +299,17 @@ void getEnlargement(int x)
 
   if (rotateZ == 1)
   {
-    scale=1.+(float)(x-xold)*0.025;
-    windows[VIEWER].top*=scale;
-    windows[VIEWER].bottom*=scale;
-    windows[VIEWER].right*=scale;
-    windows[VIEWER].left*=scale;
+    if (projectionMode == ORTHO)
+    {
+      scale=1.+(float)(x-xold)*0.025;
+      windows[VIEWER].top*=scale;
+      windows[VIEWER].bottom*=scale;
+      windows[VIEWER].right*=scale;
+      windows[VIEWER].left*=scale;
+      if (nAnnotations) scaleAnnotation(scale);
+    }
+    else
+      zmov[moveItem]+=(float)(x-xold);
   }
   if (rotateZ == -1) rotateZ=1;
   xold=x;
@@ -322,46 +329,30 @@ void setDrawingDevice(int device)
                      drawNormal3d=glNormal3d;
                      drawColor4fv=glColor4fv;
                      drawClearColor=glClearColor;
-                     drawString=printString;
-                     drawLineStipple=glLineStipple;
+                     drawSphere=gluSphere;
+                     drawCylinder=gluCylinder;
+			   drawString=printString;
+			   drawLineStipple=glLineStipple;
 			   drawLineWidth=glLineWidth;
+			   drawDisable=glDisable;
+                     break;
+    case HPGL:
+    case POSTSCRIPT: drawBegin=glBegin;
+                     drawEnd=glEnd;
+                     drawVertex3d=glVertex3d;
+                     drawVertex2d=glVertex2d;
+                     drawNormal3d=glNormal3d;
+                     drawColor4fv=glColor4fv;
+                     drawClearColor=glClearColor;
+                     drawLineStipple=glLineStipple;
+                     drawLineWidth=glLineWidth;
                      drawDisable=glDisable;
                      drawSphere=gluSphere;
                      drawCylinder=gluCylinder;
-                     drawOrtho=glOrtho;
-                     drawPopMatrix=glPopMatrix;
-                     break;
-    case HPGL:       drawBegin=hpglBegin;
-                     drawEnd=hpglEnd;
-                     drawVertex3d=hpglVertex3d;
-                     drawVertex2d=hpglVertex2d;
-                     drawNormal3d=hpglNormal3d;
-                     drawColor4fv=hpglColor4fv;
-                     drawClearColor=hpglClearColor;
-                     drawString=plotString;
-                     drawLineStipple=hpglLineStipple;
-			   drawLineWidth=hpglLineWidth;
-                     drawDisable=hpglDisable;
-                     drawSphere=hpglSphere;
-                     drawCylinder=hpglCylinder;
-                     drawOrtho=hpglOrtho;
-                     drawPopMatrix=hpglPopMatrix;
-                     break;
-    case POSTSCRIPT: drawBegin=PostscriptBegin;
-                     drawEnd=PostscriptEnd;
-                     drawVertex3d=PostscriptVertex3d;
-                     drawVertex2d=PostscriptVertex2d;
-                     drawNormal3d=PostscriptNormal3d;
-                     drawColor4fv=PostscriptColor4fv;
-                     drawClearColor=PostscriptClearColor;
-                     drawString=PostscriptString;
-                     drawLineStipple=PostscriptLineStipple;
-			   drawLineWidth=PostscriptLineWidth;
-                     drawDisable=PostscriptDisable;
-                     drawSphere=hpglSphere;
-                     drawCylinder=hpglCylinder;
-                     drawOrtho=PostscriptOrtho;
-                     drawPopMatrix=PostscriptPopMatrix;
+			   drawString=feedbackString;
+			   drawLineStipple=feedbackLineStipple;
+			   drawLineWidth=feedbackLineWidth;
+			   drawDisable=feedbackDisable;
                      break;
     case RAYTRACER:  drawBegin=raytracerBegin;
                      drawEnd=raytracerEnd;
@@ -370,14 +361,12 @@ void setDrawingDevice(int device)
                      drawNormal3d=raytracerNormal3d;
                      drawColor4fv=raytracerColor4fv;
                      drawClearColor=raytracerClearColor;
-                     drawString=printString;
-                     drawLineStipple=glLineStipple;
-			   drawLineWidth=glLineWidth;
-                     drawDisable=glDisable;
                      drawSphere=raytracerSphere;
                      drawCylinder=raytracerCylinder;
-			   drawOrtho=glOrtho;
-			   drawPopMatrix=glPopMatrix;
+			   drawString=printString;
+			   drawLineStipple=glLineStipple;
+			   drawLineWidth=glLineWidth;
+			   drawDisable=glDisable;
                      break;
   }
 }
